@@ -1,5 +1,6 @@
 /* Generic target-file-type support for the BFD library.
-   Copyright 1990, 91, 92, 93, 94, 1995 Free Software Foundation, Inc.
+   Copyright 1990, 91, 92, 93, 94, 95, 96, 97, 1998
+   Free Software Foundation, Inc.
    Written by Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -16,11 +17,12 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "bfd.h"
 #include "sysdep.h"
 #include "libbfd.h"
+#include "fnmatch.h"
 
 /*
 SECTION 
@@ -145,8 +147,15 @@ DESCRIPTION
 .  bfd_target_oasys_flavour,
 .  bfd_target_tekhex_flavour,
 .  bfd_target_srec_flavour,
+.  bfd_target_ihex_flavour,
 .  bfd_target_som_flavour,
-.  bfd_target_os9k_flavour};
+.  bfd_target_os9k_flavour,
+.  bfd_target_versados_flavour,
+.  bfd_target_msdos_flavour,
+.  bfd_target_evax_flavour
+.};
+.
+.enum bfd_endian { BFD_ENDIAN_BIG, BFD_ENDIAN_LITTLE, BFD_ENDIAN_UNKNOWN };
 .
 .{* Forward declaration.  *}
 .typedef struct bfd_link_info _bfd_link_info;
@@ -165,14 +174,14 @@ of a file.
 
 The order of bytes within the data area of a file.
 
-.  boolean byteorder_big_p;
+.  enum bfd_endian byteorder;
 
 The order of bytes within the header parts of a file.
 
-.  boolean header_byteorder_big_p;
+.  enum bfd_endian header_byteorder;
 
 A mask of all the flags which an executable may have set -
-from the set <<NO_FLAGS>>, <<HAS_RELOC>>, ...<<D_PAGED>>.
+from the set <<BFD_NO_FLAGS>>, <<HAS_RELOC>>, ...<<D_PAGED>>.
 
 .  flagword object_flags;       
 
@@ -194,10 +203,6 @@ The maximum number of characters in an archive header.
 
 .  unsigned short ar_max_namelen;
 
-The minimum alignment restriction for any section.
-
-.  unsigned int align_power_min;
-
 Entries for byte swapping for data. These are different from the other
 entry points, since they don't take a BFD asthe first argument.
 Certain other handlers could do the same.
@@ -247,7 +252,9 @@ The general target vector.
 .CAT(NAME,_close_and_cleanup),\
 .CAT(NAME,_bfd_free_cached_info),\
 .CAT(NAME,_new_section_hook),\
-.CAT(NAME,_get_section_contents)
+.CAT(NAME,_get_section_contents),\
+.CAT(NAME,_get_section_contents_in_window)
+.
 .  {* Called when the BFD is being closed to do any necessary cleanup.  *}
 .  boolean       (*_close_and_cleanup) PARAMS ((bfd *));
 .  {* Ask the BFD to free all cached information.  *}
@@ -257,18 +264,37 @@ The general target vector.
 .  {* Read the contents of a section.  *}
 .  boolean       (*_bfd_get_section_contents) PARAMS ((bfd *, sec_ptr, PTR, 
 .                                            file_ptr, bfd_size_type));
+.  boolean       (*_bfd_get_section_contents_in_window)
+.                          PARAMS ((bfd *, sec_ptr, bfd_window *,
+.                                   file_ptr, bfd_size_type));
 .
 .  {* Entry points to copy private data.  *}
 .#define BFD_JUMP_TABLE_COPY(NAME)\
 .CAT(NAME,_bfd_copy_private_bfd_data),\
-.CAT(NAME,_bfd_copy_private_section_data)
+.CAT(NAME,_bfd_merge_private_bfd_data),\
+.CAT(NAME,_bfd_copy_private_section_data),\
+.CAT(NAME,_bfd_copy_private_symbol_data),\
+.CAT(NAME,_bfd_set_private_flags),\
+.CAT(NAME,_bfd_print_private_bfd_data)\
 .  {* Called to copy BFD general private data from one object file
 .     to another.  *}
 .  boolean	 (*_bfd_copy_private_bfd_data) PARAMS ((bfd *, bfd *));
+.  {* Called to merge BFD general private data from one object file
+.     to a common output file when linking.  *}
+.  boolean	 (*_bfd_merge_private_bfd_data) PARAMS ((bfd *, bfd *));
 .  {* Called to copy BFD private section data from one object file
 .     to another.  *}
 .  boolean       (*_bfd_copy_private_section_data) PARAMS ((bfd *, sec_ptr,
 .                                                       bfd *, sec_ptr));
+.  {* Called to copy BFD private symbol data from one symbol 
+.     to another.  *}
+.  boolean       (*_bfd_copy_private_symbol_data) PARAMS ((bfd *, asymbol *,
+.							   bfd *, asymbol *));
+.  {* Called to set private backend flags *}
+.  boolean	 (*_bfd_set_private_flags) PARAMS ((bfd *, flagword));
+.
+.  {* Called to print private BFD data *}
+.  boolean       (*_bfd_print_private_bfd_data) PARAMS ((bfd *, PTR));
 .
 .  {* Core file entry points.  *}
 .#define BFD_JUMP_TABLE_CORE(NAME)\
@@ -286,7 +312,9 @@ The general target vector.
 .CAT(NAME,_construct_extended_name_table),\
 .CAT(NAME,_truncate_arname),\
 .CAT(NAME,_write_armap),\
+.CAT(NAME,_read_ar_hdr),\
 .CAT(NAME,_openr_next_archived_file),\
+.CAT(NAME,_get_elt_at_index),\
 .CAT(NAME,_generic_stat_arch_elt),\
 .CAT(NAME,_update_armap_timestamp)
 .  boolean  (*_bfd_slurp_armap) PARAMS ((bfd *));
@@ -299,7 +327,10 @@ The general target vector.
 .                              struct orl *map,
 .                              unsigned int orl_count, 
 .                              int stridx));
+.  PTR (*_bfd_read_ar_hdr_fn) PARAMS ((bfd *));
 .  bfd *    (*openr_next_archived_file) PARAMS ((bfd *arch, bfd *prev));
+.#define bfd_get_elt_at_index(b,i) BFD_SEND(b, _bfd_get_elt_at_index, (b,i))
+.  bfd *    (*_bfd_get_elt_at_index) PARAMS ((bfd *, symindex));
 .  int      (*_bfd_stat_arch_elt) PARAMS ((bfd *, struct stat *));
 .  boolean  (*_bfd_update_armap_timestamp) PARAMS ((bfd *));
 .
@@ -310,10 +341,12 @@ The general target vector.
 .CAT(NAME,_make_empty_symbol),\
 .CAT(NAME,_print_symbol),\
 .CAT(NAME,_get_symbol_info),\
-.CAT(NAME,_bfd_is_local_label),\
+.CAT(NAME,_bfd_is_local_label_name),\
 .CAT(NAME,_get_lineno),\
 .CAT(NAME,_find_nearest_line),\
-.CAT(NAME,_bfd_make_debug_symbol)
+.CAT(NAME,_bfd_make_debug_symbol),\
+.CAT(NAME,_read_minisymbols),\
+.CAT(NAME,_minisymbol_to_symbol)
 .  long  (*_bfd_get_symtab_upper_bound) PARAMS ((bfd *));
 .  long  (*_bfd_canonicalize_symtab) PARAMS ((bfd *,
 .                                             struct symbol_cache_entry **));
@@ -327,7 +360,7 @@ The general target vector.
 .                                      struct symbol_cache_entry *,
 .                                      symbol_info *));
 .#define bfd_get_symbol_info(b,p,e) BFD_SEND(b, _bfd_get_symbol_info, (b,p,e))
-.  boolean	 (*_bfd_is_local_label) PARAMS ((bfd *, asymbol *));
+.  boolean	 (*_bfd_is_local_label_name) PARAMS ((bfd *, const char *));
 .
 .  alent *    (*_get_lineno) PARAMS ((bfd *, struct symbol_cache_entry *));
 .  boolean    (*_bfd_find_nearest_line) PARAMS ((bfd *abfd,
@@ -337,10 +370,18 @@ The general target vector.
 . {* Back-door to allow format-aware applications to create debug symbols
 .    while using BFD for everything else.  Currently used by the assembler
 .    when creating COFF files.  *}
-. asymbol *  (*_bfd_make_debug_symbol) PARAMS ((
+.  asymbol *  (*_bfd_make_debug_symbol) PARAMS ((
 .       bfd *abfd,
 .       void *ptr,
 .       unsigned long size));
+.#define bfd_read_minisymbols(b, d, m, s) \
+.  BFD_SEND (b, _read_minisymbols, (b, d, m, s))
+.  long  (*_read_minisymbols) PARAMS ((bfd *, boolean, PTR *,
+.                                      unsigned int *));
+.#define bfd_minisymbol_to_symbol(b, d, m, f) \
+.  BFD_SEND (b, _minisymbol_to_symbol, (b, d, m, f))
+.  asymbol *(*_minisymbol_to_symbol) PARAMS ((bfd *, boolean, const PTR,
+.                                             asymbol *));
 .
 .  {* Routines for relocs.  *}
 .#define BFD_JUMP_TABLE_RELOCS(NAME)\
@@ -371,7 +412,8 @@ The general target vector.
 .CAT(NAME,_bfd_relax_section),\
 .CAT(NAME,_bfd_link_hash_table_create),\
 .CAT(NAME,_bfd_link_add_symbols),\
-.CAT(NAME,_bfd_final_link)
+.CAT(NAME,_bfd_final_link),\
+.CAT(NAME,_bfd_link_split_section)
 .  int        (*_bfd_sizeof_headers) PARAMS ((bfd *, boolean));
 .  bfd_byte * (*_bfd_get_relocated_section_contents) PARAMS ((bfd *,
 .                    struct bfd_link_info *, struct bfd_link_order *,
@@ -392,6 +434,9 @@ The general target vector.
 .     section of the BFD.  *}
 .  boolean (*_bfd_final_link) PARAMS ((bfd *, struct bfd_link_info *));
 .
+.  {* Should this section be split up into smaller pieces during linking.  *}
+.  boolean (*_bfd_link_split_section) PARAMS ((bfd *, struct sec *));
+.
 . {* Routines to handle dynamic symbols and relocs.  *}
 .#define BFD_JUMP_TABLE_DYNAMIC(NAME)\
 .CAT(NAME,_get_dynamic_symtab_upper_bound),\
@@ -424,30 +469,53 @@ in this structure.
    we can't intermix extern's and initializers.  */
 extern const bfd_target a29kcoff_big_vec;
 extern const bfd_target a_out_adobe_vec;
+extern const bfd_target aout_arm_big_vec;
+extern const bfd_target aout_arm_little_vec;
 extern const bfd_target aout_mips_big_vec;
 extern const bfd_target aout_mips_little_vec;
 extern const bfd_target aout0_big_vec;
 extern const bfd_target apollocoff_vec;
+extern const bfd_target armcoff_little_vec;
+extern const bfd_target armcoff_big_vec;
+extern const bfd_target armpe_little_vec;
+extern const bfd_target armpe_big_vec;
+extern const bfd_target armpei_little_vec;
+extern const bfd_target armpei_big_vec;
 extern const bfd_target b_out_vec_big_host;
 extern const bfd_target b_out_vec_little_host;
+extern const bfd_target bfd_elf64_alpha_vec;
+extern const bfd_target bfd_elf32_bigarc_vec;
+extern const bfd_target bfd_elf32_littlearc_vec;
 extern const bfd_target bfd_elf32_big_generic_vec;
 extern const bfd_target bfd_elf32_bigmips_vec;
+extern const bfd_target bfd_elf64_bigmips_vec;
+extern const bfd_target bfd_elf32_d10v_vec;
 extern const bfd_target bfd_elf32_hppa_vec;
 extern const bfd_target bfd_elf32_i386_vec;
 extern const bfd_target bfd_elf32_i860_vec;
 extern const bfd_target bfd_elf32_little_generic_vec;
 extern const bfd_target bfd_elf32_littlemips_vec;
+extern const bfd_target bfd_elf64_littlemips_vec;
+extern const bfd_target bfd_elf32_m32r_vec;
 extern const bfd_target bfd_elf32_m68k_vec;
 extern const bfd_target bfd_elf32_m88k_vec;
+extern const bfd_target bfd_elf32_mn10200_vec;
+extern const bfd_target bfd_elf32_mn10300_vec;
 extern const bfd_target bfd_elf32_powerpc_vec;
+extern const bfd_target bfd_elf32_powerpcle_vec;
+extern const bfd_target bfd_elf32_sh_vec;
+extern const bfd_target bfd_elf32_shl_vec;
 extern const bfd_target bfd_elf32_sparc_vec;
+extern const bfd_target bfd_elf32_v850_vec;
 extern const bfd_target bfd_elf64_big_generic_vec;
 extern const bfd_target bfd_elf64_little_generic_vec;
 extern const bfd_target bfd_elf64_sparc_vec;
 extern const bfd_target demo_64_vec;
 extern const bfd_target ecoff_big_vec;
 extern const bfd_target ecoff_little_vec;
+extern const bfd_target ecoff_biglittle_vec;
 extern const bfd_target ecoffalpha_little_vec;
+extern const bfd_target evax_alpha_vec;
 extern const bfd_target h8300coff_vec;
 extern const bfd_target h8500coff_vec;
 extern const bfd_target host_aout_vec;
@@ -457,22 +525,36 @@ extern const bfd_target som_vec;
 extern const bfd_target i386aout_vec;
 extern const bfd_target i386bsd_vec;
 extern const bfd_target i386dynix_vec;
+extern const bfd_target i386freebsd_vec;
 extern const bfd_target i386os9k_vec;
 extern const bfd_target i386coff_vec;
+extern const bfd_target bfd_powerpc_pe_vec;
+extern const bfd_target bfd_powerpcle_pe_vec;
+extern const bfd_target bfd_powerpc_pei_vec;
+extern const bfd_target bfd_powerpcle_pei_vec;
+extern const bfd_target i386pe_vec;
+extern const bfd_target i386pei_vec;
 extern const bfd_target go32coff_vec;
+extern const bfd_target go32stubbedcoff_vec;
 extern const bfd_target i386linux_vec;
 extern const bfd_target i386lynx_aout_vec;
 extern const bfd_target i386lynx_coff_vec;
 extern const bfd_target i386mach3_vec;
+extern const bfd_target i386msdos_vec;
 extern const bfd_target i386netbsd_vec;
+extern const bfd_target i860coff_vec;
 extern const bfd_target icoff_big_vec;
 extern const bfd_target icoff_little_vec;
 extern const bfd_target ieee_vec;
+extern const bfd_target m68kaux_coff_vec;
 extern const bfd_target m68kcoff_vec;
 extern const bfd_target m68kcoffun_vec;
+extern const bfd_target m68klinux_vec;
 extern const bfd_target m68klynx_aout_vec;
 extern const bfd_target m68klynx_coff_vec;
 extern const bfd_target m68knetbsd_vec;
+extern const bfd_target m68ksysvcoff_vec;
+extern const bfd_target m68k4knetbsd_vec;
 extern const bfd_target m88kbcs_vec;
 extern const bfd_target m88kmach3_vec;
 extern const bfd_target newsos3_vec;
@@ -483,16 +565,25 @@ extern const bfd_target nlm32_powerpc_vec;
 extern const bfd_target pc532netbsd_vec;
 extern const bfd_target oasys_vec;
 extern const bfd_target pc532machaout_vec;
+extern const bfd_target ppcboot_vec;
 extern const bfd_target riscix_vec;
+extern const bfd_target pmac_xcoff_vec;
 extern const bfd_target rs6000coff_vec;
 extern const bfd_target shcoff_vec;
 extern const bfd_target shlcoff_vec;
+extern const bfd_target shcoff_small_vec;
+extern const bfd_target shlcoff_small_vec;
+extern const bfd_target sparcle_aout_vec;
+extern const bfd_target sparclinux_vec;
 extern const bfd_target sparclynx_aout_vec;
 extern const bfd_target sparclynx_coff_vec;
 extern const bfd_target sparcnetbsd_vec;
 extern const bfd_target sparccoff_vec;
 extern const bfd_target sunos_big_vec;
 extern const bfd_target tekhex_vec;
+extern const bfd_target tic30_aout_vec;
+extern const bfd_target tic30_coff_vec;
+extern const bfd_target versados_vec;
 extern const bfd_target we32kcoff_vec;
 extern const bfd_target w65_vec;
 extern const bfd_target z8kcoff_vec;
@@ -504,12 +595,16 @@ extern const bfd_target symbolsrec_vec;
 /* binary is always included.  */
 extern const bfd_target binary_vec;
 
+/* ihex is always included.  */
+extern const bfd_target ihex_vec;
+
 /* All of the xvecs for core files.  */
 extern const bfd_target aix386_core_vec;
 extern const bfd_target cisco_core_vec;
 extern const bfd_target hpux_core_vec;
 extern const bfd_target hppabsd_core_vec;
 extern const bfd_target irix_core_vec;
+extern const bfd_target netbsd_core_vec;
 extern const bfd_target osf_core_vec;
 extern const bfd_target sco_core_vec;
 extern const bfd_target trad_core_vec;
@@ -546,16 +641,32 @@ const bfd_target * const bfd_target_vector[] = {
 	   --enable-targets=all, objdump or gdb should be able to examine
 	   the file even if we don't recognize the machine type.  */
 	&bfd_elf32_big_generic_vec,
+#ifdef BFD64
+	&bfd_elf64_alpha_vec,
+#endif
+	&bfd_elf32_bigarc_vec,
 	&bfd_elf32_bigmips_vec,
+#ifdef BFD64
+	&bfd_elf64_bigmips_vec,
+#endif
+	&bfd_elf32_d10v_vec,
 	&bfd_elf32_hppa_vec,
 	&bfd_elf32_i386_vec,
 	&bfd_elf32_i860_vec,
 	&bfd_elf32_little_generic_vec,
+	&bfd_elf32_littlearc_vec,
 	&bfd_elf32_littlemips_vec,
+#ifdef BFD64
+	&bfd_elf64_littlemips_vec,
+#endif
+	&bfd_elf32_m32r_vec,
+	&bfd_elf32_mn10200_vec,
+	&bfd_elf32_mn10300_vec,
 	&bfd_elf32_m68k_vec,
 	&bfd_elf32_m88k_vec,
 	&bfd_elf32_sparc_vec,
 	&bfd_elf32_powerpc_vec,
+	&bfd_elf32_v850_vec,
 #ifdef BFD64			/* No one seems to use this.  */
 	&bfd_elf64_big_generic_vec,
 	&bfd_elf64_little_generic_vec,
@@ -571,8 +682,10 @@ const bfd_target * const bfd_target_vector[] = {
 #endif
 	&ecoff_big_vec,
 	&ecoff_little_vec,
-#if 0
+	&ecoff_biglittle_vec,
+#ifdef BFD64
 	&ecoffalpha_little_vec,
+	&evax_alpha_vec,
 #endif
 	&h8300coff_vec,
 	&h8500coff_vec,
@@ -591,7 +704,14 @@ const bfd_target * const bfd_target_vector[] = {
 	&i386aout_vec,
 	&i386bsd_vec,
 	&i386coff_vec,
+	&i386freebsd_vec,
+	&i860coff_vec,
+	&bfd_powerpc_pe_vec,
+	&bfd_powerpcle_pe_vec,
+	&bfd_powerpc_pei_vec,
+	&bfd_powerpcle_pei_vec,
 	&go32coff_vec,
+	&go32stubbedcoff_vec,
 #if 0
 	/* Since a.out files lack decent magic numbers, no way to recognize
 	   which kind of a.out file it is.  */
@@ -603,16 +723,31 @@ const bfd_target * const bfd_target_vector[] = {
 	/* No distinguishing features for Mach 3 executables.  */
 	&i386mach3_vec,
 #endif
+	&i386msdos_vec,
 	&i386netbsd_vec,
 	&i386os9k_vec,
+	&i386pe_vec,
+	&i386pei_vec,
+	&armcoff_little_vec,
+	&armcoff_big_vec,
+	&armpe_little_vec,
+	&armpe_big_vec,
+	&armpei_little_vec,
+	&armpei_big_vec,
 	&icoff_big_vec,
 	&icoff_little_vec,
 	&ieee_vec,
 	&m68kcoff_vec,
 	&m68kcoffun_vec,
+#if 0
+	/* Since a.out files lack decent magic numbers, no way to recognize
+	   which kind of a.out file it is.  */
+	&m68klinux_vec,
+#endif
 	&m68klynx_aout_vec,
 	&m68klynx_coff_vec,
 	&m68knetbsd_vec,
+	&m68ksysvcoff_vec,
 	&m88kbcs_vec,
 	&m88kmach3_vec,
 	&newsos3_vec,
@@ -632,19 +767,33 @@ const bfd_target * const bfd_target_vector[] = {
 #endif
 	&pc532machaout_vec,
 #if 0
-	/* We have no way of distinguishing this from other a.out variants */
+	/* We have no way of distinguishing these from other a.out variants */
+	&aout_arm_big_vec,
+	&aout_arm_little_vec,
 	&riscix_vec,
+#endif
+#if 0
+	/* This has the same magic number as RS/6000. */
+	&pmac_xcoff_vec,
 #endif
 	&rs6000coff_vec,
+	&ppcboot_vec,
 	&shcoff_vec,
 	&shlcoff_vec,
+	&shcoff_small_vec,
+	&shlcoff_small_vec,
+	&sparcle_aout_vec,
+	&sparclinux_vec,
 	&sparclynx_aout_vec,
 	&sparclynx_coff_vec,
 	&sparcnetbsd_vec,
 	&sunos_big_vec,
 	&aout0_big_vec,
 	&tekhex_vec,
+	&tic30_aout_vec,
+	&tic30_coff_vec,
 	&we32kcoff_vec,
+	&versados_vec,
 	&z8kcoff_vec,
 
 #endif /* not SELECT_VECS */
@@ -652,9 +801,12 @@ const bfd_target * const bfd_target_vector[] = {
 /* Always support S-records, for convenience.  */
 	&srec_vec,
 	&symbolsrec_vec,
-
+/* And tekhex */
+	&tekhex_vec,
 /* Likewise for binary output.  */
 	&binary_vec,
+/* Likewise for ihex.  */
+	&ihex_vec,
 
 /* Add any required traditional-core-file-handler.  */
 
@@ -687,7 +839,7 @@ const bfd_target * const bfd_target_vector[] = {
 /* bfd_default_vector[0] contains either the address of the default vector,
    if there is one, or zero if there isn't.  */
 
-const bfd_target * const bfd_default_vector[] = {
+const bfd_target *bfd_default_vector[] = {
 #ifdef DEFAULT_VECTOR
 	&DEFAULT_VECTOR,
 #endif
@@ -698,6 +850,88 @@ const bfd_target * const bfd_default_vector[] = {
    names of the matching targets in an array.  This variable is the maximum
    number of entries that the array could possibly need.  */
 const size_t _bfd_target_vector_entries = sizeof(bfd_target_vector)/sizeof(*bfd_target_vector);
+
+/* This array maps configuration triplets onto BFD vectors.  */
+
+struct targmatch
+{
+  /* The configuration triplet.  */
+  const char *triplet;
+  /* The BFD vector.  If this is NULL, then the vector is found by
+     searching forward for the next structure with a non NULL vector
+     field.  */
+  const bfd_target *vector;
+};
+
+/* targmatch.h is built by Makefile out of config.bfd.  */
+static const struct targmatch bfd_target_match[] = {
+#include "targmatch.h"
+  { NULL, NULL }
+};
+
+static const bfd_target *find_target PARAMS ((const char *));
+
+/* Find a target vector, given a name or configuration triplet.  */
+
+static const bfd_target *
+find_target (name)
+     const char *name;
+{
+  const bfd_target * const *target;
+  const struct targmatch *match;
+
+  for (target = &bfd_target_vector[0]; *target != NULL; target++)
+    if (strcmp (name, (*target)->name) == 0)
+      return *target;
+
+  /* If we couldn't match on the exact name, try matching on the
+     configuration triplet.  FIXME: We should run the triplet through
+     config.sub first, but that is hard.  */
+  for (match = &bfd_target_match[0]; match->triplet != NULL; match++)
+    {
+      if (fnmatch (match->triplet, name, 0) == 0)
+	{
+	  while (match->vector == NULL)
+	    ++match;
+	  return match->vector;
+	  break;
+	}
+    }
+
+  bfd_set_error (bfd_error_invalid_target);
+  return NULL;
+}
+
+/*
+FUNCTION
+	bfd_set_default_target
+
+SYNOPSIS
+	boolean bfd_set_default_target (const char *name);
+
+DESCRIPTION
+	Set the default target vector to use when recognizing a BFD.
+	This takes the name of the target, which may be a BFD target
+	name or a configuration triplet.
+*/
+
+boolean
+bfd_set_default_target (name)
+     const char *name;
+{
+  const bfd_target *target;
+
+  if (bfd_default_vector[0] != NULL
+      && strcmp (name, bfd_default_vector[0]->name) == 0)
+    return true;
+
+  target = find_target (name);
+  if (target == NULL)
+    return false;
+
+  bfd_default_vector[0] = target;
+  return true;
+}
 
 /*
 FUNCTION
@@ -720,32 +954,38 @@ DESCRIPTION
 
 const bfd_target *
 bfd_find_target (target_name, abfd)
-     CONST char *target_name;
+     const char *target_name;
      bfd *abfd;
 {
-  const bfd_target * const *target;
-  extern char *getenv ();
-  CONST char *targname = (target_name ? target_name : 
-			  (CONST char *) getenv ("GNUTARGET"));
+  const char *targname;
+  const bfd_target *target;
+
+  if (target_name != NULL)
+    targname = target_name;
+  else
+    targname = getenv ("GNUTARGET");
 
   /* This is safe; the vector cannot be null */
-  if (targname == NULL || !strcmp (targname, "default")) {
-    abfd->target_defaulted = true;
-    return abfd->xvec = bfd_target_vector[0];
-  }
+  if (targname == NULL || strcmp (targname, "default") == 0)
+    {
+      abfd->target_defaulted = true;
+      if (bfd_default_vector[0] != NULL)
+	abfd->xvec = bfd_default_vector[0];
+      else
+	abfd->xvec = bfd_target_vector[0];
+      return abfd->xvec;
+    }
 
   abfd->target_defaulted = false;
 
-  for (target = &bfd_target_vector[0]; *target != NULL; target++) {
-    if (!strcmp (targname, (*target)->name))
-      return abfd->xvec = *target;
-  }
+  target = find_target (targname);
+  if (target == NULL)
+    return NULL;
 
-  bfd_set_error (bfd_error_invalid_target);
-  return NULL;
+  abfd->xvec = target;
+  return target;
 }
 
-
 /*
 FUNCTION
 	bfd_target_list
@@ -764,7 +1004,7 @@ const char **
 bfd_target_list ()
 {
   int vec_length= 0;
-#ifdef NATIVE_HPPAHPUX_COMPILER
+#if defined (HOST_HPPAHPUX) && ! defined (__STDC__)
   /* The native compiler on the HP9000/700 has a bug which causes it
      to loop endlessly when compiling this file.  This avoids it.  */
   volatile
@@ -778,10 +1018,8 @@ bfd_target_list ()
   name_ptr = name_list = (CONST char **)
     bfd_zmalloc ((vec_length + 1) * sizeof (char **));
 
-  if (name_list == NULL) {
-    bfd_set_error (bfd_error_no_memory);
+  if (name_list == NULL)
     return NULL;
-  }
 
   for (target = &bfd_target_vector[0]; *target != NULL; target++)
     *(name_ptr++) = (*target)->name;
