@@ -1,6 +1,5 @@
 /* BFD backend for Extended Tektronix Hex Format  objects.
-   Copyright (C) 1992, 1993 Free Software Foundation, Inc.
-
+   Copyright (C) 1992, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
    Written by Steve Chamberlain of Cygnus Support <sac@cygnus.com>.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -17,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /*
 SUBSECTION
@@ -72,6 +71,7 @@ serial protocol, so big files are unlikely, so we keep a list of 8k chunks
 #include "bfd.h"
 #include "sysdep.h"
 #include "libbfd.h"
+#include "libiberty.h"
 
 typedef struct
   {
@@ -87,23 +87,46 @@ typedef struct tekhex_symbol_struct
 
   } tekhex_symbol_type;
 
-static char digs[] = "0123456789ABCDEF";
+static const char digs[] = "0123456789ABCDEF";
 
 static char sum_block[256];
 
-/* Horrible ascii dependent macros for converting between hex and
-   binary */
-
-#define CHARS_IN_SET 256
-static char hex_value[CHARS_IN_SET];
-
 #define NOT_HEX 20
-#define NIBBLE(x) hex_value[x]
+#define NIBBLE(x) hex_value(x)
 #define HEX(buffer) ((NIBBLE((buffer)[0])<<4) + NIBBLE((buffer)[1]))
 #define TOHEX(d,x) \
 (d)[1] = digs[(x) & 0xf]; \
 (d)[0] = digs[((x)>>4)&0xf];
-#define	ISHEX(x)  (hex_value[x] != NOT_HEX)
+#define	ISHEX(x)  hex_p(x)
+
+static void tekhex_init PARAMS ((void));
+static bfd_vma getvalue PARAMS ((char **));
+static void tekhex_print_symbol
+ PARAMS ((bfd *, PTR, asymbol *, bfd_print_symbol_type));
+static void tekhex_get_symbol_info PARAMS ((bfd *, asymbol *, symbol_info *));
+static asymbol *tekhex_make_empty_symbol PARAMS ((bfd *));
+static int tekhex_sizeof_headers PARAMS ((bfd *, boolean));
+static boolean tekhex_write_object_contents PARAMS ((bfd *));
+static void out PARAMS ((bfd *, int, char *, char *));
+static void writesym PARAMS ((char **, CONST char *));
+static void writevalue PARAMS ((char **, bfd_vma));
+static boolean tekhex_set_section_contents
+ PARAMS ((bfd*, sec_ptr, PTR, file_ptr, bfd_size_type));
+static boolean tekhex_set_arch_mach
+ PARAMS ((bfd *, enum bfd_architecture, unsigned long));
+static boolean tekhex_get_section_contents
+ PARAMS ((bfd *, asection *, PTR, file_ptr, bfd_size_type));
+static void move_section_contents
+ PARAMS ((bfd *, asection *, PTR, file_ptr, bfd_size_type, boolean));
+static const bfd_target *tekhex_object_p PARAMS ((bfd *));
+static boolean tekhex_mkobject PARAMS ((bfd *));
+static long tekhex_get_symtab_upper_bound PARAMS ((bfd *));
+static long tekhex_get_symtab PARAMS ((bfd *, asymbol **));
+static void pass_over PARAMS ((bfd *, void (*)(bfd*, int, char *)));
+static void first_phase PARAMS ((bfd *, int, char *));
+static void insert_byte PARAMS ((bfd *, int, bfd_vma));
+static struct data_struct *find_chunk PARAMS ((bfd *, bfd_vma));
+static unsigned int getsym PARAMS ((char *, char **));
 
 /*
 Here's an example
@@ -223,24 +246,8 @@ tekhex_init ()
 
   if (inited == false)
     {
-
       inited = true;
-
-      for (i = 0; i < CHARS_IN_SET; i++)
-	{
-	  hex_value[i] = NOT_HEX;
-	}
-
-      for (i = 0; i < 10; i++)
-	{
-	  hex_value[i + '0'] = i;
-
-	}
-      for (i = 0; i < 6; i++)
-	{
-	  hex_value[i + 'a'] = i + 10;
-	  hex_value[i + 'A'] = i + 10;
-	}
+      hex_init ();
       val = 0;
       for (i = 0; i < 10; i++)
 	{
@@ -305,13 +312,13 @@ getvalue (srcp)
 {
   char *src = *srcp;
   bfd_vma value = 0;
-  unsigned int len = hex_value[*src++];
+  unsigned int len = hex_value(*src++);
 
   if (len == 0)
     len = 16;
   while (len--)
     {
-      value = value << 4 | hex_value[*src++];
+      value = value << 4 | hex_value(*src++);
     }
   *srcp = src;
   return value;
@@ -324,7 +331,7 @@ getsym (dstp, srcp)
 {
   char *src = *srcp;
   unsigned int i;
-  unsigned int len = hex_value[*src++];
+  unsigned int len = hex_value(*src++);
 
   if (len == 0)
     len = 16;
@@ -335,7 +342,7 @@ getsym (dstp, srcp)
   return len;
 }
 
-struct data_struct *
+static struct data_struct *
 find_chunk (abfd, vma)
      bfd *abfd;
      bfd_vma vma;
@@ -350,12 +357,14 @@ find_chunk (abfd, vma)
   if (!d)
     {
       char *sname = bfd_alloc (abfd, 12);
-      asection *s;
 
       /* No chunk for this address, so make one up */
       d = (struct data_struct *)
 	bfd_alloc (abfd, sizeof (struct data_struct));
 
+      if (!sname || !d)
+	return NULL;
+
       memset (d->chunk_init, 0, CHUNK_MASK + 1);
       memset (d->chunk_data, 0, CHUNK_MASK + 1);
       d->next = abfd->tdata.tekhex_data->data;
@@ -383,10 +392,10 @@ insert_byte (abfd, value, addr)
 static void
 first_phase (abfd, type, src)
      bfd *abfd;
-     char type;
+     int type;
      char *src;
 {
-  asection *section = &bfd_abs_section;
+  asection *section = bfd_abs_section_ptr;
   int len;
   char sym[17];			/* A symbol can only be 16chars long */
 
@@ -414,6 +423,8 @@ first_phase (abfd, type, src)
 	{
 	  char *n = bfd_alloc (abfd, len + 1);
 
+	  if (!n)
+	    abort();		/* FIXME */
 	  memcpy (n, sym, len + 1);
 	  section = bfd_make_section (abfd, n);
 	}
@@ -421,15 +432,13 @@ first_phase (abfd, type, src)
 	{
 	  switch (*src)
 	    {
-	      asection *s;
-
 	    case '1':		/* section range */
 	      src++;
 	      section->vma = getvalue (&src);
 	      section->_raw_size = getvalue (&src) - section->vma;
 	      section->flags = SEC_HAS_CONTENTS | SEC_LOAD | SEC_ALLOC;
 	      break;
-
+	    case '0':
 	    case '2':
 	    case '3':
 	    case '4':
@@ -443,6 +452,8 @@ first_phase (abfd, type, src)
 					       sizeof (tekhex_symbol_type));
 		char type = (*src);
 
+		if (!new)
+		  abort();	/* FIXME */
 		new->symbol.the_bfd = abfd;
 		src++;
 		abfd->symcount++;
@@ -451,6 +462,8 @@ first_phase (abfd, type, src)
 		abfd->tdata.tekhex_data->symbols = new;
 		len = getsym (sym, &src);
 		new->symbol.name = bfd_alloc (abfd, len + 1);
+		if (!new->symbol.name)
+		  abort();	/* FIXME */
 		memcpy ((char *) (new->symbol.name), sym, len + 1);
 		new->symbol.section = section;
 		if (type <= '4')
@@ -468,21 +481,21 @@ first_phase (abfd, type, src)
    record.  */
 
 static void
- pass_over (abfd, func)
+pass_over (abfd, func)
      bfd *abfd;
-     void (*func) ();
+     void (*func) PARAMS ((bfd *, int, char *));
 {
   unsigned int chars_on_line;
   boolean eof = false;
 
   /* To the front of the file */
-  bfd_seek (abfd, (file_ptr) 0, SEEK_SET);
+  if (bfd_seek (abfd, (file_ptr) 0, SEEK_SET) != 0)
+    abort ();
   while (eof == false)
     {
       char buffer[MAXCHUNK];
       char *src = buffer;
       char type;
-      bfd_vma address = 0;
 
       /* Find first '%' */
       eof = (boolean) (bfd_read (src, 1, 1, abfd) != 1);
@@ -495,7 +508,8 @@ static void
       src++;
 
       /* Fetch the type and the length and the checksum */
-      bfd_read (src, 1, 5, abfd);
+      if (bfd_read (src, 1, 5, abfd) != 5)
+	abort (); /* FIXME */
 
       type = src[2];
 
@@ -504,7 +518,8 @@ static void
 
       chars_on_line = HEX (src) - 5;	/* Already read five char */
 
-      bfd_read (src, 1, chars_on_line, abfd);
+      if (bfd_read (src, 1, chars_on_line, abfd) != chars_on_line)
+	abort (); /* FIXME */
       src[chars_on_line] = 0;	/* put a null at the end */
 
       func (abfd, type, src);
@@ -512,11 +527,10 @@ static void
 
 }
 
-unsigned int
+static long
 tekhex_get_symtab (abfd, table)
      bfd *abfd;
      asymbol **table;
-
 {
   tekhex_symbol_type *p = abfd->tdata.tekhex_data->symbols;
   unsigned int c = bfd_get_symcount (abfd);
@@ -531,7 +545,7 @@ tekhex_get_symtab (abfd, table)
   return bfd_get_symcount (abfd);
 }
 
-unsigned int
+static long
 tekhex_get_symtab_upper_bound (abfd)
      bfd *abfd;
 {
@@ -545,33 +559,34 @@ tekhex_mkobject (abfd)
 {
   tdata_type *tdata = (tdata_type *) bfd_alloc (abfd, sizeof (tdata_type));
 
+  if (!tdata)
+    return false;
   abfd->tdata.tekhex_data = tdata;
   tdata->type = 1;
   tdata->head = (tekhex_data_list_type *) NULL;
   tdata->symbols = (struct tekhex_symbol_struct *) NULL;
   tdata->data = (struct data_struct *) NULL;
   return true;
-
 }
 
 /*
   Return true if the file looks like it's in TekHex format. Just look
   for a percent sign and some hex digits */
 
-static bfd_target *
+static const bfd_target *
 tekhex_object_p (abfd)
      bfd *abfd;
 {
   char b[4];
-  asection *section;
 
   tekhex_init ();
 
-  bfd_seek (abfd, (file_ptr) 0, SEEK_SET);
-  bfd_read (b, 1, 4, abfd);
+  if (bfd_seek (abfd, (file_ptr) 0, SEEK_SET) != 0
+      || bfd_read (b, 1, 4, abfd) != 4)
+    return NULL;
 
   if (b[0] != '%' || !ISHEX (b[1]) || !ISHEX (b[2]) || !ISHEX (b[3]))
-    return (bfd_target *) NULL;
+    return (const bfd_target *) NULL;
 
   tekhex_mkobject (abfd);
 
@@ -579,7 +594,7 @@ tekhex_object_p (abfd)
   return abfd->xvec;
 }
 
-static boolean
+static void
 move_section_contents (abfd, section, locationp, offset, count, get)
      bfd *abfd;
      asection *section;
@@ -627,6 +642,7 @@ move_section_contents (abfd, section, locationp, offset, count, get)
     }
 
 }
+
 static boolean
 tekhex_get_section_contents (abfd, section, locationp, offset, count)
      bfd *abfd;
@@ -644,7 +660,7 @@ tekhex_get_section_contents (abfd, section, locationp, offset, count)
     return false;
 }
 
-boolean
+static boolean
 tekhex_set_arch_mach (abfd, arch, machine)
      bfd *abfd;
      enum bfd_architecture arch;
@@ -761,13 +777,14 @@ writesym (dst, sym)
 static void
 out (abfd, type, start, end)
      bfd *abfd;
-     char type;
+     int type;
      char *start;
      char *end;
 {
   int sum = 0;
   char *s;
   char front[6];
+  bfd_size_type wrlen;
 
   front[0] = '%';
   TOHEX (front + 1, end - start + 5);
@@ -775,17 +792,19 @@ out (abfd, type, start, end)
 
   for (s = start; s < end; s++)
     {
-      sum += sum_block[*s];
+      sum += sum_block[(unsigned char) *s];
     }
 
-  sum += sum_block[front[1]];	/*  length */
-  sum += sum_block[front[2]];
-  sum += sum_block[front[3]];	/* type */
+  sum += sum_block[(unsigned char) front[1]];	/*  length */
+  sum += sum_block[(unsigned char) front[2]];
+  sum += sum_block[(unsigned char) front[3]];	/* type */
   TOHEX (front + 4, sum);
-  bfd_write (front, 1, 6, abfd);
+  if (bfd_write (front, 1, 6, abfd) != 6)
+    abort ();
   end[0] = '\n';
-  bfd_write (start, 1, end - start + 1, abfd);
-
+  wrlen = end - start + 1;
+  if (bfd_write (start, 1, wrlen, abfd) != wrlen)
+    abort ();
 }
 
 static boolean
@@ -795,8 +814,6 @@ tekhex_write_object_contents (abfd)
   int bytes_written;
   char buffer[100];
   asymbol **p;
-  tdata_type *tdata = abfd->tdata.tekhex_data;
-  tekhex_data_list_type *list;
   asection *s;
   struct data_struct *d;
 
@@ -851,60 +868,64 @@ tekhex_write_object_contents (abfd)
     }
 
   /* And the symbols */
-  for (p = abfd->outsymbols; *p; p++)
+  if (abfd->outsymbols)
     {
-      int section_code = bfd_decode_symclass (*p);
+      for (p = abfd->outsymbols; *p; p++)
+	{
+	  int section_code = bfd_decode_symclass (*p);
 
-      if (section_code != '?')
-	{			/* do not include debug symbols */
-	  asymbol *s = *p;
-	  char *dst = buffer;
+	  if (section_code != '?')
+	    {			/* do not include debug symbols */
+	      asymbol *s = *p;
+	      char *dst = buffer;
 
-	  writesym (&dst, s->section->name);
+	      writesym (&dst, s->section->name);
 
-	  switch (section_code)
-	    {
-	    case 'A':
-	      *dst++ = '2';
-	      break;
-	    case 'a':
-	      *dst++ = '6';
-	      break;
-	    case 'D':
-	    case 'B':
-	    case 'O':
-	      *dst++ = '4';
-	      break;
-	    case 'd':
-	    case 'b':
-	    case 'o':
-	      *dst++ = '8';
-	      break;
-	    case 'T':
-	      *dst++ = '3';
-	      break;
-	    case 't':
-	      *dst++ = '7';
-	      break;
-	    case 'C':
-	    case 'U':
-	      bfd_error = wrong_format;
-	      return false;
+	      switch (section_code)
+		{
+		case 'A':
+		  *dst++ = '2';
+		  break;
+		case 'a':
+		  *dst++ = '6';
+		  break;
+		case 'D':
+		case 'B':
+		case 'O':
+		  *dst++ = '4';
+		  break;
+		case 'd':
+		case 'b':
+		case 'o':
+		  *dst++ = '8';
+		  break;
+		case 'T':
+		  *dst++ = '3';
+		  break;
+		case 't':
+		  *dst++ = '7';
+		  break;
+		case 'C':
+		case 'U':
+		  bfd_set_error (bfd_error_wrong_format);
+		  return false;
+		}
+
+	      writesym (&dst, s->name);
+	      writevalue (&dst, s->value + s->section->vma);
+	      out (abfd, '3', buffer, dst);
 	    }
-
-	  writesym (&dst, s->name);
-	  writevalue (&dst, s->value + s->section->vma);
-	  out (abfd, '3', buffer, dst);
 	}
     }
 
   /* And the terminator */
-  bfd_write ("%7081010\n", 1, 9, abfd);
+  if (bfd_write ("%0781010\n", 1, 9, abfd) != 9)
+    abort ();
   return true;
 }
 
 static int
-  tekhex_sizeof_headers (abfd, exec)
+tekhex_sizeof_headers (abfd, exec)
      bfd *abfd;
      boolean exec;
 
@@ -919,6 +940,8 @@ tekhex_make_empty_symbol (abfd)
   tekhex_symbol_type *new =
   (tekhex_symbol_type *) bfd_zalloc (abfd, sizeof (struct tekhex_symbol_struct));
 
+  if (!new)
+    return NULL;
   new->symbol.the_bfd = abfd;
   new->prev = (struct tekhex_symbol_struct *) NULL;
   return &(new->symbol);
@@ -963,50 +986,42 @@ tekhex_print_symbol (ignore_abfd, filep, symbol, how)
     }
 }
 
-#define FOO PROTO
-#define tekhex_new_section_hook (FOO(boolean, (*), (bfd *, asection *)))bfd_true
-#define tekhex_get_reloc_upper_bound (FOO(unsigned int, (*),(bfd*, asection *)))bfd_false
-#define tekhex_canonicalize_reloc (FOO(unsigned int, (*),(bfd*,asection *, arelent **, asymbol **))) bfd_0
+#define	tekhex_close_and_cleanup _bfd_generic_close_and_cleanup
+#define tekhex_bfd_free_cached_info _bfd_generic_bfd_free_cached_info
+#define tekhex_new_section_hook _bfd_generic_new_section_hook
 
-#define tekhex_openr_next_archived_file (FOO(bfd *, (*), (bfd*,bfd*))) bfd_nullvoidptr
-#define tekhex_find_nearest_line (FOO(boolean, (*),(bfd*,asection*,asymbol**,bfd_vma, CONST char**, CONST char**, unsigned int *))) bfd_false
-#define tekhex_generic_stat_arch_elt  (FOO(int, (*), (bfd *,struct stat *))) bfd_0
+#define tekhex_bfd_is_local_label_name bfd_generic_is_local_label_name
+#define tekhex_get_lineno _bfd_nosymbols_get_lineno
+#define tekhex_find_nearest_line _bfd_nosymbols_find_nearest_line
+#define tekhex_bfd_make_debug_symbol _bfd_nosymbols_bfd_make_debug_symbol
+#define tekhex_read_minisymbols _bfd_generic_read_minisymbols
+#define tekhex_minisymbol_to_symbol _bfd_generic_minisymbol_to_symbol
 
-#define tekhex_core_file_failing_command (char *(*)())(bfd_nullvoidptr)
-#define tekhex_core_file_failing_signal (int (*)())bfd_0
-#define tekhex_core_file_matches_executable_p (FOO(boolean, (*),(bfd*, bfd*)))bfd_false
-#define tekhex_slurp_armap bfd_true
-#define tekhex_slurp_extended_name_table bfd_true
-#define tekhex_truncate_arname (void (*)())bfd_nullvoidptr
-#define tekhex_write_armap  (FOO( boolean, (*),(bfd *, unsigned int, struct orl *, unsigned int, int))) bfd_nullvoidptr
-#define tekhex_get_lineno (struct lineno_cache_entry *(*)())bfd_nullvoidptr
-#define	tekhex_close_and_cleanup	bfd_generic_close_and_cleanup
-#define tekhex_bfd_debug_info_start bfd_void
-#define tekhex_bfd_debug_info_end bfd_void
-#define tekhex_bfd_debug_info_accumulate  (FOO(void, (*), (bfd *,	 asection *))) bfd_void
-#define tekhex_bfd_get_relocated_section_contents bfd_generic_get_relocated_section_contents
+#define tekhex_bfd_get_relocated_section_contents \
+  bfd_generic_get_relocated_section_contents
 #define tekhex_bfd_relax_section bfd_generic_relax_section
-#define tekhex_bfd_seclet_link bfd_generic_seclet_link
-#define tekhex_bfd_reloc_type_lookup \
-  ((CONST struct reloc_howto_struct *(*) PARAMS ((bfd *, bfd_reloc_code_real_type))) bfd_nullvoidptr)
-#define tekhex_bfd_make_debug_symbol \
-  ((asymbol *(*) PARAMS ((bfd *, void *, unsigned long))) bfd_nullvoidptr)
+#define tekhex_bfd_link_hash_table_create _bfd_generic_link_hash_table_create
+#define tekhex_bfd_link_add_symbols _bfd_generic_link_add_symbols
+#define tekhex_bfd_final_link _bfd_generic_final_link
+#define tekhex_bfd_link_split_section _bfd_generic_link_split_section
 
-bfd_target tekhex_vec =
+#define tekhex_get_section_contents_in_window \
+  _bfd_generic_get_section_contents_in_window
+
+const bfd_target tekhex_vec =
 {
   "tekhex",			/* name */
   bfd_target_tekhex_flavour,
-  true,				/* target byte order */
-  true,				/* target headers byte order */
+  BFD_ENDIAN_UNKNOWN,		/* target byte order */
+  BFD_ENDIAN_UNKNOWN,		/* target headers byte order */
   (EXEC_P |			/* object flags */
-   HAS_SYMS | HAS_LINENO | HAS_DEBUG | HAS_RELOC | HAS_LOCALS | DYNAMIC |
+   HAS_SYMS | HAS_LINENO | HAS_DEBUG | HAS_RELOC | HAS_LOCALS |
    WP_TEXT | D_PAGED),
   (SEC_CODE | SEC_DATA | SEC_ROM | SEC_HAS_CONTENTS
    | SEC_ALLOC | SEC_LOAD | SEC_RELOC),	/* section flags */
   0,				/* leading underscore */
   ' ',				/* ar_pad_char */
   16,				/* ar_max_namelen */
-  1,				/* minimum alignment */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
   bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* data */
@@ -1017,8 +1032,8 @@ bfd_target tekhex_vec =
   {
     _bfd_dummy_target,
     tekhex_object_p,		/* bfd_check_format */
-    (struct bfd_target * (*)()) bfd_nullvoidptr,
-    (struct bfd_target * (*)()) bfd_nullvoidptr,
+    _bfd_dummy_target,
+    _bfd_dummy_target,
   },
   {
     bfd_false,
@@ -1032,6 +1047,16 @@ bfd_target tekhex_vec =
     _bfd_write_archive_contents,
     bfd_false,
   },
-  JUMP_TABLE (tekhex),
+
+  BFD_JUMP_TABLE_GENERIC (tekhex),
+  BFD_JUMP_TABLE_COPY (_bfd_generic),
+  BFD_JUMP_TABLE_CORE (_bfd_nocore),
+  BFD_JUMP_TABLE_ARCHIVE (_bfd_noarchive),
+  BFD_JUMP_TABLE_SYMBOLS (tekhex),
+  BFD_JUMP_TABLE_RELOCS (_bfd_norelocs),
+  BFD_JUMP_TABLE_WRITE (tekhex),
+  BFD_JUMP_TABLE_LINK (tekhex),
+  BFD_JUMP_TABLE_DYNAMIC (_bfd_nodynamic),
+
   (PTR) 0
 };
