@@ -1,6 +1,7 @@
 /* Get info from stack frames;
    convert between frames, blocks, functions and pc values.
-   Copyright 1986, 1987, 1988, 1989, 1991, 1994 Free Software Foundation, Inc.
+   Copyright 1986, 1987, 1988, 1989, 1991, 1994, 1995, 1996, 1997
+             Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -16,7 +17,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "defs.h"
 #include "symtab.h"
@@ -49,7 +50,8 @@ inside_entry_file (addr)
 #if CALL_DUMMY_LOCATION == AT_ENTRY_POINT
   /* Do not stop backtracing if the pc is in the call dummy
      at the entry point.  */
-  if (PC_IN_CALL_DUMMY (addr, 0, 0))
+/* FIXME: Won't always work with zeros for the last two arguments */
+  if (PC_IN_CALL_DUMMY (addr, 0, 0))	
     return 0;
 #endif
   return (addr >= symfile_objfile -> ei.entry_file_lowpc &&
@@ -72,6 +74,25 @@ CORE_ADDR pc;
     return 1;
   if (symfile_objfile == 0)
     return 0;
+
+  /* If the addr range is not set up at symbol reading time, set it up now.
+     This is for FRAME_CHAIN_VALID_ALTERNATE. I do this for coff, because
+     it is unable to set it up and symbol reading time. */
+
+  if (symfile_objfile -> ei.main_func_lowpc == INVALID_ENTRY_LOWPC &&
+      symfile_objfile -> ei.main_func_highpc == INVALID_ENTRY_HIGHPC)
+    {
+      struct symbol *mainsym;
+
+      mainsym = lookup_symbol ("main", NULL, VAR_NAMESPACE, NULL, NULL);
+      if (mainsym && SYMBOL_CLASS(mainsym) == LOC_BLOCK)
+        {
+          symfile_objfile->ei.main_func_lowpc = 
+	    BLOCK_START (SYMBOL_BLOCK_VALUE (mainsym));
+          symfile_objfile->ei.main_func_highpc = 
+	    BLOCK_END (SYMBOL_BLOCK_VALUE (mainsym));
+        }
+    }
   return (symfile_objfile -> ei.main_func_lowpc  <= pc &&
 	  symfile_objfile -> ei.main_func_highpc > pc);
 }
@@ -95,6 +116,7 @@ CORE_ADDR pc;
 #if CALL_DUMMY_LOCATION == AT_ENTRY_POINT
   /* Do not stop backtracing if the pc is in the call dummy
      at the entry point.  */
+/* FIXME: Won't always work with zeros for the last two arguments */
   if (PC_IN_CALL_DUMMY (pc, 0, 0))
     return 0;
 #endif
@@ -226,9 +248,10 @@ frameless_look_for_prologue (frame)
      struct frame_info *frame;
 {
   CORE_ADDR func_start, after_prologue;
-  func_start = (get_pc_function_start (frame->pc) + FUNCTION_START_OFFSET);
+  func_start = get_pc_function_start (frame->pc);
   if (func_start)
     {
+      func_start += FUNCTION_START_OFFSET;
       after_prologue = func_start;
 #ifdef SKIP_PROLOGUE_FRAMELESS_P
       /* This is faster, since only care whether there *is* a prologue,
@@ -239,6 +262,12 @@ frameless_look_for_prologue (frame)
 #endif
       return after_prologue == func_start;
     }
+  else if (frame->pc == 0)
+    /* A frame with a zero PC is usually created by dereferencing a NULL
+       function pointer, normally causing an immediate core dump of the
+       inferior. Mark function as frameless, as the inferior has no chance
+       of setting up a stack frame.  */
+    return 1;
   else
     /* If we can't find the start of the function, we don't really
        know whether the function is frameless, but we should be able
@@ -508,27 +537,32 @@ get_frame_function (frame)
   return block_function (bl);
 }
 
+
 /* Return the blockvector immediately containing the innermost lexical block
-   containing the specified pc value, or 0 if there is none.
+   containing the specified pc value and section, or 0 if there is none.
    PINDEX is a pointer to the index value of the block.  If PINDEX
    is NULL, we don't pass this information back to the caller.  */
 
 struct blockvector *
-blockvector_for_pc (pc, pindex)
+blockvector_for_pc_sect (pc, section, pindex, symtab)
      register CORE_ADDR pc;
+     struct sec *section;
      int *pindex;
+     struct symtab *symtab;
+     
 {
   register struct block *b;
   register int bot, top, half;
-  register struct symtab *s;
   struct blockvector *bl;
 
-  /* First search all symtabs for one whose file contains our pc */
-  s = find_pc_symtab (pc);
-  if (s == 0)
-    return 0;
+  if (symtab == 0)	/* if no symtab specified by caller */
+    {
+      /* First search all symtabs for one whose file contains our pc */
+      if ((symtab = find_pc_sect_symtab (pc, section)) == 0)
+	return 0;
+    }
 
-  bl = BLOCKVECTOR (s);
+  bl = BLOCKVECTOR (symtab);
   b = BLOCKVECTOR_BLOCK (bl, 0);
 
   /* Then search that symtab for the smallest block that wins.  */
@@ -560,45 +594,80 @@ blockvector_for_pc (pc, pindex)
 	}
       bot--;
     }
-
   return 0;
 }
 
-/* Return the innermost lexical block containing the specified pc value,
-   or 0 if there is none.  */
+/* Return the blockvector immediately containing the innermost lexical block
+   containing the specified pc value, or 0 if there is none.
+   Backward compatibility, no section.  */
+
+struct blockvector *
+blockvector_for_pc (pc, pindex)
+     register CORE_ADDR pc;
+     int *pindex;
+{
+  return blockvector_for_pc_sect (pc, find_pc_mapped_section (pc),
+				  pindex, NULL);
+}
+
+/* Return the innermost lexical block containing the specified pc value
+   in the specified section, or 0 if there is none.  */
 
 struct block *
-block_for_pc (pc)
+block_for_pc_sect (pc, section)
      register CORE_ADDR pc;
+     struct sec *section;
 {
   register struct blockvector *bl;
   int index;
 
-  bl = blockvector_for_pc (pc, &index);
+  bl = blockvector_for_pc_sect (pc, section, &index, NULL);
   if (bl)
     return BLOCKVECTOR_BLOCK (bl, index);
   return 0;
 }
 
-/* Return the function containing pc value PC.
+/* Return the innermost lexical block containing the specified pc value,
+   or 0 if there is none.  Backward compatibility, no section.  */
+
+struct block *
+block_for_pc (pc)
+     register CORE_ADDR pc;
+{
+  return block_for_pc_sect (pc, find_pc_mapped_section (pc));
+}
+
+/* Return the function containing pc value PC in section SECTION.
    Returns 0 if function is not known.  */
 
 struct symbol *
-find_pc_function (pc)
+find_pc_sect_function (pc, section)
      CORE_ADDR pc;
+     struct sec *section;
 {
-  register struct block *b = block_for_pc (pc);
+  register struct block *b = block_for_pc_sect (pc, section);
   if (b == 0)
     return 0;
   return block_function (b);
 }
 
+/* Return the function containing pc value PC.
+   Returns 0 if function is not known.  Backward compatibility, no section */
+
+struct symbol *
+find_pc_function (pc)
+     CORE_ADDR pc;
+{
+  return find_pc_sect_function (pc, find_pc_mapped_section (pc));
+}
+
 /* These variables are used to cache the most recent result
  * of find_pc_partial_function. */
 
-static CORE_ADDR cache_pc_function_low = 0;
-static CORE_ADDR cache_pc_function_high = 0;
-static char *cache_pc_function_name = 0;
+static CORE_ADDR   cache_pc_function_low     = 0;
+static CORE_ADDR   cache_pc_function_high    = 0;
+static char       *cache_pc_function_name    = 0;
+static struct sec *cache_pc_function_section = NULL;
 
 /* Clear cache, e.g. when symbol table is discarded. */
 
@@ -608,33 +677,37 @@ clear_pc_function_cache()
   cache_pc_function_low = 0;
   cache_pc_function_high = 0;
   cache_pc_function_name = (char *)0;
+  cache_pc_function_section = NULL;
 }
 
 /* Finds the "function" (text symbol) that is smaller than PC but
-   greatest of all of the potential text symbols.  Sets *NAME and/or
-   *ADDRESS conditionally if that pointer is non-null.  If ENDADDR is
-   non-null, then set *ENDADDR to be the end of the function
-   (exclusive), but passing ENDADDR as non-null means that the
-   function might cause symbols to be read.  This function either
+   greatest of all of the potential text symbols in SECTION.  Sets
+   *NAME and/or *ADDRESS conditionally if that pointer is non-null.
+   If ENDADDR is non-null, then set *ENDADDR to be the end of the
+   function (exclusive), but passing ENDADDR as non-null means that
+   the function might cause symbols to be read.  This function either
    succeeds or fails (not halfway succeeds).  If it succeeds, it sets
    *NAME, *ADDRESS, and *ENDADDR to real information and returns 1.
-   If it fails, it sets *NAME, *ADDRESS, and *ENDADDR to zero
-   and returns 0.  */
+   If it fails, it sets *NAME, *ADDRESS, and *ENDADDR to zero and
+   returns 0.  */
 
 int
-find_pc_partial_function (pc, name, address, endaddr)
-     CORE_ADDR pc;
-     char **name;
+find_pc_sect_partial_function (pc, section, name, address, endaddr)
+     CORE_ADDR  pc;
+     asection  *section;
+     char     **name;
      CORE_ADDR *address;
      CORE_ADDR *endaddr;
 {
   struct partial_symtab *pst;
-  struct symbol *f;
+  struct symbol         *f;
   struct minimal_symbol *msymbol;
   struct partial_symbol *psb;
-  struct obj_section *sec;
+  struct obj_section    *osect;
 
-  if (pc >= cache_pc_function_low && pc < cache_pc_function_high)
+
+  if (pc >= cache_pc_function_low && pc < cache_pc_function_high &&
+      section == cache_pc_function_section)
     goto return_cached_value;
 
   /* If sigtramp is in the u area, it counts as a function (especially
@@ -642,16 +715,16 @@ find_pc_partial_function (pc, name, address, endaddr)
 #if defined SIGTRAMP_START
   if (IN_SIGTRAMP (pc, (char *)NULL))
     {
-      cache_pc_function_low = SIGTRAMP_START;
-      cache_pc_function_high = SIGTRAMP_END;
-      cache_pc_function_name = "<sigtramp>";
-
+      cache_pc_function_low     = SIGTRAMP_START (pc);
+      cache_pc_function_high    = SIGTRAMP_END (pc);
+      cache_pc_function_name    = "<sigtramp>";
+      cache_pc_function_section = section;
       goto return_cached_value;
     }
 #endif
 
-  msymbol = lookup_minimal_symbol_by_pc (pc);
-  pst = find_pc_psymtab (pc);
+  msymbol = lookup_minimal_symbol_by_pc_section (pc, section);
+  pst = find_pc_sect_psymtab (pc, section);
   if (pst)
     {
       /* Need to read the symbols to get a good value for the end address.  */
@@ -667,15 +740,16 @@ find_pc_partial_function (pc, name, address, endaddr)
 	{
 	  /* Checking whether the msymbol has a larger value is for the
 	     "pathological" case mentioned in print_frame_info.  */
-	  f = find_pc_function (pc);
+	  f = find_pc_sect_function (pc, section);
 	  if (f != NULL
 	      && (msymbol == NULL
 		  || (BLOCK_START (SYMBOL_BLOCK_VALUE (f))
 		      >= SYMBOL_VALUE_ADDRESS (msymbol))))
 	    {
-	      cache_pc_function_low = BLOCK_START (SYMBOL_BLOCK_VALUE (f));
-	      cache_pc_function_high = BLOCK_END (SYMBOL_BLOCK_VALUE (f));
-	      cache_pc_function_name = SYMBOL_NAME (f);
+	      cache_pc_function_low     = BLOCK_START (SYMBOL_BLOCK_VALUE (f));
+	      cache_pc_function_high    = BLOCK_END   (SYMBOL_BLOCK_VALUE (f));
+	      cache_pc_function_name    = SYMBOL_NAME (f);
+	      cache_pc_function_section = section;
 	      goto return_cached_value;
 	    }
 	}
@@ -684,7 +758,7 @@ find_pc_partial_function (pc, name, address, endaddr)
 	  /* Now that static symbols go in the minimal symbol table, perhaps
 	     we could just ignore the partial symbols.  But at least for now
 	     we use the partial or minimal symbol, whichever is larger.  */
-	  psb = find_pc_psymbol (pst, pc);
+	  psb = find_pc_sect_psymbol (pst, pc, section);
 
 	  if (psb
 	      && (msymbol == NULL ||
@@ -707,9 +781,9 @@ find_pc_partial_function (pc, name, address, endaddr)
      of the text seg doesn't appear to be part of the last function in the
      text segment.  */
 
-  sec = find_pc_section (pc);
+  osect = find_pc_sect_section (pc, section);
 
-  if (!sec)
+  if (!osect)
     msymbol = NULL;
 
   /* Must be in the minimal symbol table.  */
@@ -725,37 +799,20 @@ find_pc_partial_function (pc, name, address, endaddr)
       return 0;
     }
 
-  /* See if we're in a transfer table for Sun shared libs.
-
-     Note the hack for Sun shared library transfer tables creates
-     problems for single stepping through the return path from a shared
-     library call if the return path includes trampoline code.
-
-     I don't really understand the reasoning behind the magic handling
-     for mst_trampoline symbols.  */
-
-#ifdef INHIBIT_SUNSOLIB_TRANSFER_TABLE_HACK
-    cache_pc_function_low = SYMBOL_VALUE_ADDRESS (msymbol);
-#else
-  if (msymbol -> type == mst_text || msymbol -> type == mst_file_text)
-    cache_pc_function_low = SYMBOL_VALUE_ADDRESS (msymbol);
-  else
-    /* It is a transfer table for Sun shared libraries.  */
-    cache_pc_function_low = pc - FUNCTION_START_OFFSET;
-#endif
-
-  cache_pc_function_name = SYMBOL_NAME (msymbol);
+  cache_pc_function_low     = SYMBOL_VALUE_ADDRESS (msymbol);
+  cache_pc_function_name    = SYMBOL_NAME (msymbol);
+  cache_pc_function_section = section;
 
   /* Use the lesser of the next minimal symbol, or the end of the section, as
      the end of the function.  */
 
   if (SYMBOL_NAME (msymbol + 1) != NULL
-      && SYMBOL_VALUE_ADDRESS (msymbol + 1) < sec->endaddr)
+      && SYMBOL_VALUE_ADDRESS (msymbol + 1) < osect->endaddr)
     cache_pc_function_high = SYMBOL_VALUE_ADDRESS (msymbol + 1);
   else
     /* We got the start address from the last msymbol in the objfile.
        So the end address is the end of the section.  */
-    cache_pc_function_high = sec->endaddr;
+    cache_pc_function_high = osect->endaddr;
 
  return_cached_value:
   if (address)
@@ -767,6 +824,23 @@ find_pc_partial_function (pc, name, address, endaddr)
   return 1;
 }
 
+/* Backward compatibility, no section argument */
+
+int
+find_pc_partial_function (pc, name, address, endaddr)
+     CORE_ADDR  pc;
+     char     **name;
+     CORE_ADDR *address;
+     CORE_ADDR *endaddr;
+{
+  asection     *section;
+
+  section = find_pc_overlay (pc);
+  if (pc_in_unmapped_range (pc, section))
+    pc = overlay_mapped_address (pc, section);
+  return find_pc_sect_partial_function (pc, section, name, address, endaddr);
+}
+
 /* Return the innermost stack frame executing inside of BLOCK,
    or NULL if there is no such frame.  If BLOCK is NULL, just return NULL.  */
 
@@ -847,6 +921,251 @@ sigtramp_saved_pc (frame)
 }
 #endif /* SIGCONTEXT_PC_OFFSET */
 
+#ifdef USE_GENERIC_DUMMY_FRAMES
+
+/*
+ * GENERIC DUMMY FRAMES
+ * 
+ * The following code serves to maintain the dummy stack frames for
+ * inferior function calls (ie. when gdb calls into the inferior via
+ * call_function_by_hand).  This code saves the machine state before 
+ * the call in host memory, so we must maintain an independant stack 
+ * and keep it consistant etc.  I am attempting to make this code 
+ * generic enough to be used by many targets.
+ *
+ * The cheapest and most generic way to do CALL_DUMMY on a new target
+ * is probably to define CALL_DUMMY to be empty, CALL_DUMMY_LENGTH to zero,
+ * and CALL_DUMMY_LOCATION to AT_ENTRY.  Then you must remember to define
+ * PUSH_RETURN_ADDRESS, because no call instruction will be being
+ * executed by the target.
+ */
+
+static struct dummy_frame *dummy_frame_stack = NULL;
+
+/* Function: find_dummy_frame(pc, fp, sp)
+   Search the stack of dummy frames for one matching the given PC, FP and SP.
+   This is the work-horse for pc_in_call_dummy and read_register_dummy     */
+
+char * 
+generic_find_dummy_frame (pc, fp)
+     CORE_ADDR pc;
+     CORE_ADDR fp;
+{
+  struct dummy_frame * dummyframe;
+
+  if (pc != entry_point_address ())
+    return 0;
+
+  for (dummyframe = dummy_frame_stack; dummyframe != NULL;
+       dummyframe = dummyframe->next)
+    if (fp == dummyframe->fp || fp == dummyframe->sp)
+      /* The frame in question lies between the saved fp and sp, inclusive */
+      return dummyframe->regs;
+
+  return 0;
+}
+
+/* Function: pc_in_call_dummy (pc, fp)
+   Return true if this is a dummy frame created by gdb for an inferior call */
+
+int
+generic_pc_in_call_dummy (pc, fp)
+     CORE_ADDR pc;
+     CORE_ADDR fp;
+{
+  /* if find_dummy_frame succeeds, then PC is in a call dummy */
+  return (generic_find_dummy_frame (pc, fp) != 0);
+}
+
+/* Function: read_register_dummy 
+   Find a saved register from before GDB calls a function in the inferior */
+
+CORE_ADDR
+generic_read_register_dummy (pc, fp, regno)
+     CORE_ADDR pc;
+     CORE_ADDR fp;
+     int regno;
+{
+  char *dummy_regs = generic_find_dummy_frame (pc, fp);
+
+  if (dummy_regs)
+    return extract_address (&dummy_regs[REGISTER_BYTE (regno)],
+			    REGISTER_RAW_SIZE(regno));
+  else
+    return 0;
+}
+
+/* Save all the registers on the dummy frame stack.  Most ports save the
+   registers on the target stack.  This results in lots of unnecessary memory
+   references, which are slow when debugging via a serial line.  Instead, we
+   save all the registers internally, and never write them to the stack.  The
+   registers get restored when the called function returns to the entry point,
+   where a breakpoint is laying in wait.  */
+
+void
+generic_push_dummy_frame ()
+{
+  struct dummy_frame *dummy_frame;
+  CORE_ADDR fp = (get_current_frame ())->frame;
+
+  /* check to see if there are stale dummy frames, 
+     perhaps left over from when a longjump took us out of a 
+     function that was called by the debugger */
+
+  dummy_frame = dummy_frame_stack;
+  while (dummy_frame)
+    if (dummy_frame->fp INNER_THAN fp)	/* stale -- destroy! */
+      {
+	dummy_frame_stack = dummy_frame->next;
+	free (dummy_frame);
+	dummy_frame = dummy_frame_stack;
+      }
+    else
+      dummy_frame = dummy_frame->next;
+
+  dummy_frame = xmalloc (sizeof (struct dummy_frame));
+  dummy_frame->pc   = read_register (PC_REGNUM);
+  dummy_frame->sp   = read_register (SP_REGNUM);
+  dummy_frame->fp   = fp;
+  read_register_bytes (0, dummy_frame->regs, REGISTER_BYTES);
+  dummy_frame->next = dummy_frame_stack;
+  dummy_frame_stack = dummy_frame;
+}
+
+/* Function: pop_dummy_frame
+   Restore the machine state from a saved dummy stack frame. */
+
+void
+generic_pop_dummy_frame ()
+{
+  struct dummy_frame *dummy_frame = dummy_frame_stack;
+
+  /* FIXME: what if the first frame isn't the right one, eg..
+     because one call-by-hand function has done a longjmp into another one? */
+
+  if (!dummy_frame)
+    error ("Can't pop dummy frame!");
+  dummy_frame_stack = dummy_frame->next;
+  write_register_bytes (0, dummy_frame->regs, REGISTER_BYTES);
+  free (dummy_frame);
+}
+
+/* Function: frame_chain_valid 
+   Returns true for a user frame or a call_function_by_hand dummy frame,
+   and false for the CRT0 start-up frame.  Purpose is to terminate backtrace */
+ 
+int
+generic_frame_chain_valid (fp, fi)
+     CORE_ADDR fp;
+     struct frame_info *fi;
+{
+  if (PC_IN_CALL_DUMMY(FRAME_SAVED_PC(fi), fp, fp))
+    return 1;   /* don't prune CALL_DUMMY frames */
+  else          /* fall back to default algorithm (see frame.h) */
+    return (fp != 0
+	    && (fi->frame INNER_THAN fp || fi->frame == fp)
+	    && !inside_entry_file (FRAME_SAVED_PC(fi)));
+}
+ 
+/* Function: get_saved_register
+   Find register number REGNUM relative to FRAME and put its (raw,
+   target format) contents in *RAW_BUFFER.  
+
+   Set *OPTIMIZED if the variable was optimized out (and thus can't be
+   fetched).  Note that this is never set to anything other than zero
+   in this implementation.
+
+   Set *LVAL to lval_memory, lval_register, or not_lval, depending on
+   whether the value was fetched from memory, from a register, or in a
+   strange and non-modifiable way (e.g. a frame pointer which was
+   calculated rather than fetched).  We will use not_lval for values
+   fetched from generic dummy frames.
+
+   Set *ADDRP to the address, either in memory on as a REGISTER_BYTE
+   offset into the registers array.  If the value is stored in a dummy
+   frame, set *ADDRP to zero.
+
+   To use this implementation, define a function called
+   "get_saved_register" in your target code, which simply passes all
+   of its arguments to this function.
+
+   The argument RAW_BUFFER must point to aligned memory.  */
+
+void
+generic_get_saved_register (raw_buffer, optimized, addrp, frame, regnum, lval)
+     char *raw_buffer;
+     int *optimized;
+     CORE_ADDR *addrp;
+     struct frame_info *frame;
+     int regnum;
+     enum lval_type *lval;
+{
+  struct frame_saved_regs fsr;
+
+  if (!target_has_registers)
+    error ("No registers.");
+
+  /* Normal systems don't optimize out things with register numbers.  */
+  if (optimized != NULL)
+    *optimized = 0;
+
+  if (addrp)		/* default assumption: not found in memory */
+    *addrp = 0;
+
+  /* Note: since the current frame's registers could only have been
+     saved by frames INTERIOR TO the current frame, we skip examining
+     the current frame itself: otherwise, we would be getting the
+     previous frame's registers which were saved by the current frame.  */
+
+  while (frame && ((frame = frame->next) != NULL))
+    {
+      if (PC_IN_CALL_DUMMY (frame->pc, frame->frame, frame->frame))
+	{
+	  if (lval)			/* found it in a CALL_DUMMY frame */
+	    *lval = not_lval;
+	  if (raw_buffer)
+	    memcpy (raw_buffer, 
+		    generic_find_dummy_frame (frame->pc, frame->frame) + 
+		    REGISTER_BYTE (regnum),
+		    REGISTER_RAW_SIZE (regnum));
+	      return;
+	}
+
+      FRAME_FIND_SAVED_REGS(frame, fsr);
+      if (fsr.regs[regnum] != 0)
+	{
+	  if (lval)			/* found it saved on the stack */
+	    *lval = lval_memory;
+	  if (regnum == SP_REGNUM)
+	    {
+	      if (raw_buffer)		/* SP register treated specially */
+		store_address (raw_buffer, REGISTER_RAW_SIZE (regnum), 
+			       fsr.regs[regnum]);
+	    }
+	  else
+	    {
+	      if (addrp)		/* any other register */
+		*addrp = fsr.regs[regnum];
+	      if (raw_buffer)
+		read_memory (fsr.regs[regnum], raw_buffer, 
+			     REGISTER_RAW_SIZE (regnum));
+	    }
+	  return;
+	}
+    }
+
+  /* If we get thru the loop to this point, it means the register was
+     not saved in any frame.  Return the actual live-register value.  */
+
+  if (lval)				/* found it in a live register */
+    *lval = lval_register;
+  if (addrp)
+    *addrp = REGISTER_BYTE (regnum);
+  if (raw_buffer)
+    read_register_gen (regnum, raw_buffer);
+}
+#endif /* USE_GENERIC_DUMMY_FRAMES */
+
 void
 _initialize_blockframe ()
 {
