@@ -1,4 +1,4 @@
-/* Native support for linux, for GDB, the GNU debugger.
+/* Native support for GNU/Linux, for GDB, the GNU debugger.
    Copyright (C) 1986, 1987, 1989, 1992 Free Software Foundation, Inc.
 
 This file is part of GDB.
@@ -17,6 +17,9 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+#ifndef XM_LINUX_H
+#define XM_LINUX_H
+
 #define HOST_BYTE_ORDER LITTLE_ENDIAN
 
 #define HAVE_TERMIOS
@@ -29,3 +32,5 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Need R_OK etc, but USG isn't defined.  */
 #include <unistd.h>
+
+#endif	/* #ifndef XM_LINUX_H */
