@@ -1,5 +1,5 @@
 /* Memory-access and commands for "inferior" process, for GDB.
-   Copyright 1986, 1987, 1988, 1989, 1991, 1992, 1995 Free Software Foundation, Inc.
+   Copyright 1986, 1987, 1988, 1989, 1991, 1992, 1995, 1996 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -19,7 +19,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "defs.h"
 #include <signal.h>
-#include <sys/param.h>
 #include "gdb_string.h"
 #include "symtab.h"
 #include "gdbtypes.h"
@@ -31,6 +30,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include "gdbcore.h"
 #include "target.h"
 #include "language.h"
+#include "symfile.h"
+
+/* Functions exported for general use: */
+
+void nofp_registers_info PARAMS ((char *, int));
+
+void all_registers_info PARAMS ((char *, int));
+
+void registers_info PARAMS ((char *, int));
+
+/* Local functions: */
 
 static void continue_command PARAMS ((char *, int));
 
@@ -48,13 +58,9 @@ static void float_info PARAMS ((char *, int));
 
 static void detach_command PARAMS ((char *, int));
 
-static void nofp_registers_info PARAMS ((char *, int));
-
-static void all_registers_info PARAMS ((char *, int));
-
-static void registers_info PARAMS ((char *, int));
-
+#if !defined (DO_REGISTERS_INFO)
 static void do_registers_info PARAMS ((int, int));
+#endif
 
 static void unset_environment_command PARAMS ((char *, int));
 
@@ -82,6 +88,10 @@ static void step_command PARAMS ((char *, int));
 
 static void run_command PARAMS ((char *, int));
 
+#ifdef CALL_DUMMY_BREAKPOINT_OFFSET
+static void breakpoint_auto_delete_contents PARAMS ((PTR));
+#endif
+
 #define ERROR_NO_INFERIOR \
    if (!target_has_execution) error ("The program is not being run.");
 
@@ -187,7 +197,7 @@ run_command (args, from_tty)
 
   dont_repeat ();
 
-  if (inferior_pid)
+  if (inferior_pid != 0 && target_has_execution)
     {
       if (
 	  !query ("The program being debugged has been started already.\n\
@@ -200,6 +210,8 @@ Start it from the beginning? "))
 
   exec_file = (char *) get_exec_file (0);
 
+  do_run_cleanups (NULL);
+
   /* The exec file is re-read every time we do a generic_mourn_inferior, so
      we just have to worry about the symbol file.  */
   reread_symbols ();
@@ -437,6 +449,21 @@ jump_command (arg, from_tty)
 	}
     }
 
+  if (sfn != NULL) 
+    {
+      fixup_symbol_section (sfn, 0);
+      if (section_is_overlay (SYMBOL_BFD_SECTION (sfn)) && 
+	  !section_is_mapped (SYMBOL_BFD_SECTION (sfn)))
+	{
+	  if (!query ("WARNING!!!  Destination is in unmapped overlay!  Jump anyway? "))
+	    {
+	      error ("Not confirmed.");
+	      /* NOTREACHED */
+	    }
+	}
+    }
+
+
   addr = sal.pc;
 
   if (from_tty)
@@ -500,13 +527,18 @@ signal_command (signum_exp, from_tty)
 
 /* Call breakpoint_auto_delete on the current contents of the bpstat
    pointed to by arg (which is really a bpstat *).  */
-void
+
+#ifdef CALL_DUMMY_BREAKPOINT_OFFSET
+
+static void
 breakpoint_auto_delete_contents (arg)
      PTR arg;
 {
   breakpoint_auto_delete (*(bpstat *)arg);
 }
 
+#endif	/* CALL_DUMMY_BREAKPOINT_OFFSET */
+
 /* Execute a "stack dummy", a piece of code stored in the stack
    by the debugger to be executed in the inferior.
 
@@ -546,13 +578,13 @@ run_stack_dummy (addr, buffer)
     struct breakpoint *bpt;
     struct symtab_and_line sal;
 
+    INIT_SAL (&sal);	/* initialize to zeroes */
 #if CALL_DUMMY_LOCATION != AT_ENTRY_POINT
     sal.pc = addr - CALL_DUMMY_START_OFFSET + CALL_DUMMY_BREAKPOINT_OFFSET;
 #else
     sal.pc = CALL_DUMMY_ADDRESS ();
 #endif
-    sal.symtab = NULL;
-    sal.line = 0;
+    sal.section = find_pc_overlay (sal.pc);
 
     /* Set up a FRAME for the dummy frame so we can pass it to
        set_momentary_breakpoint.  We need to give the breakpoint a
@@ -731,7 +763,7 @@ finish_command (arg, from_tty)
       val = value_being_returned (value_type, stop_registers,
 	      using_struct_return (value_of_variable (function, NULL),
 				   funcaddr,
-				   value_type,
+				   check_typedef (value_type),
 		BLOCK_GCC_COMPILED (SYMBOL_BLOCK_VALUE (function))));
 
       printf_filtered ("Value returned is $%d = ", record_latest_value (val));
@@ -922,9 +954,13 @@ path_command (dirname, from_tty)
      int from_tty;
 {
   char *exec_path;
-
+  char *env;
   dont_repeat ();
-  exec_path = strsave (get_in_environ (inferior_environ, path_var_name));
+  env = get_in_environ (inferior_environ, path_var_name);
+  /* Can be null if path is not set */
+  if (!env)
+    env = "";
+  exec_path = strsave (env);
   mod_path (dirname, &exec_path);
   set_in_environ (inferior_environ, path_var_name, exec_path);
   free (exec_path);
@@ -973,6 +1009,11 @@ do_registers_info (regnum, fpregs)
 	  continue;
       }
 
+      /* If the register name is empty, it is undefined for this
+	 processor, so don't display anything.  */
+      if (reg_names[i] == NULL || *(reg_names[i]) == '\0')
+	continue;
+
       fputs_filtered (reg_names[i], gdb_stdout);
       print_spaces_filtered (15 - strlen (reg_names[i]), gdb_stdout);
 
@@ -1010,7 +1051,11 @@ do_registers_info (regnum, fpregs)
 
 	  printf_filtered ("\t(raw 0x");
 	  for (j = 0; j < REGISTER_RAW_SIZE (i); j++)
-	    printf_filtered ("%02x", (unsigned char)raw_buffer[j]);
+	    {
+	      register int idx = TARGET_BYTE_ORDER == BIG_ENDIAN ? j
+		: REGISTER_RAW_SIZE (i) - 1 - j;
+	      printf_filtered ("%02x", (unsigned char)raw_buffer[idx]);
+	    }
 	  printf_filtered (")");
 	}
 
@@ -1018,7 +1063,7 @@ do_registers_info (regnum, fpregs)
 
       /* Else if virtual format is too long for printf,
 	 print in hex a byte at a time.  */
-      else if (REGISTER_VIRTUAL_SIZE (i) > sizeof (long))
+      else if (REGISTER_VIRTUAL_SIZE (i) > (int) sizeof (long))
 	{
 	  register int j;
 	  printf_filtered ("0x");
@@ -1028,10 +1073,10 @@ do_registers_info (regnum, fpregs)
       /* Else print as integer in hex and in decimal.  */
       else
 	{
-	  val_print (REGISTER_VIRTUAL_TYPE (i), raw_buffer, 0,
+	  val_print (REGISTER_VIRTUAL_TYPE (i), virtual_buffer, 0,
 		     gdb_stdout, 'x', 1, 0, Val_pretty_default);
 	  printf_filtered ("\t");
-	  val_print (REGISTER_VIRTUAL_TYPE (i), raw_buffer, 0,
+	  val_print (REGISTER_VIRTUAL_TYPE (i), virtual_buffer, 0,
 		     gdb_stdout,   0, 1, 0, Val_pretty_default);
 	}
 
@@ -1046,7 +1091,7 @@ do_registers_info (regnum, fpregs)
 }
 #endif /* no DO_REGISTERS_INFO.  */
 
-static void
+void
 registers_info (addr_exp, fpregs)
      char *addr_exp;
      int fpregs;
@@ -1056,6 +1101,8 @@ registers_info (addr_exp, fpregs)
 
   if (!target_has_registers)
     error ("The program has no registers now.");
+  if (selected_frame == NULL)
+    error ("No selected frame.");
 
   if (!addr_exp)
     {
@@ -1089,7 +1136,7 @@ found:
     } while (*addr_exp != '\0');
 }
 
-static void
+void
 all_registers_info (addr_exp, from_tty)
      char *addr_exp;
      int from_tty;
@@ -1097,13 +1144,14 @@ all_registers_info (addr_exp, from_tty)
   registers_info (addr_exp, 1);
 }
 
-static void
+void
 nofp_registers_info (addr_exp, from_tty)
      char *addr_exp;
      int from_tty;
 {
   registers_info (addr_exp, 0);
 }
+
 
 /*
  * TODO:
@@ -1126,6 +1174,10 @@ attach_command (args, from_tty)
      char *args;
      int from_tty;
 {
+#ifdef SOLIB_ADD
+  extern int auto_solib_add;
+#endif
+
   dont_repeat ();			/* Not for the faint of heart */
 
   if (target_has_execution)
@@ -1151,16 +1203,19 @@ attach_command (args, from_tty)
   clear_proceed_status ();
   stop_soon_quietly = 1;
 
-#ifndef MACH
-  /* Mach 3 does not generate any traps when attaching to inferior,
-     and to set up frames we can do this.  */
-
+  /* No traps are generated when attaching to inferior under Mach 3
+     or GNU hurd.  */
+#ifndef ATTACH_NO_WAIT
   wait_for_inferior ();
 #endif
 
 #ifdef SOLIB_ADD
-  /* Add shared library symbols from the newly attached process, if any.  */
-  SOLIB_ADD ((char *)0, from_tty, (struct target_ops *)0);
+  if (auto_solib_add)
+    {
+      /* Add shared library symbols from the newly attached process, if any.  */
+      SOLIB_ADD ((char *)0, from_tty, (struct target_ops *)0);
+      re_enable_breakpoints_in_shlibs ();
+    }
 #endif
 
   normal_stop ();
@@ -1220,7 +1275,7 @@ _initialize_infcmd ()
   add_show_from_set
     (add_set_cmd ("args", class_run, var_string_noescape, (char *)&inferior_args,
 		  
-"Set arguments to give program being debugged when it is started.\n\
+"Set argument list to give program being debugged when it is started.\n\
 Follow this command with any number of args, to be passed to the program.",
 		  &setlist),
      &showlist);
