@@ -1,5 +1,5 @@
 /* Low level interface to ptrace, for GDB when running under Unix.
-   Copyright 1986, 1987, 1989, 1991, 1992 Free Software Foundation, Inc.
+   Copyright 1986-87, 1989, 1991-92, 1995, 1998 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "defs.h"
 #include "frame.h"
@@ -25,17 +25,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "serial.h"
 #include "terminal.h"
 #include "target.h"
-#include "thread.h"
+#include "gdbthread.h"
 
+#include "gdb_string.h"
 #include <signal.h>
 #include <fcntl.h>
-
-#if !defined (HAVE_TERMIOS) && !defined (HAVE_TERMIO) && !defined (HAVE_SGTTY) && !defined (__GO32__)
-#define HAVE_SGTTY
-#endif
-
-#if defined (HAVE_TERMIOS)
-#include <termios.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
@@ -43,6 +38,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define PROCESS_GROUP_TYPE pid_t
 #endif
 
+#ifdef HAVE_TERMIO
+#define PROCESS_GROUP_TYPE int
+#endif
+
 #ifdef HAVE_SGTTY
 #ifdef SHORT_PGRP
 /* This is only used for the ultra.  Does it have pid_t?  */
@@ -52,17 +51,19 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 #endif /* sgtty */
 
+#if defined (SIGIO) && defined (FASYNC) && defined (FD_SET) && defined (F_SETOWN)
+static void
+handle_sigio PARAMS ((int));
+#endif
+
+static void
+pass_signal PARAMS ((int));
+
 static void
 kill_command PARAMS ((char *, int));
 
 static void
 terminal_ours_1 PARAMS ((int));
-
-/* Nonzero if we are debugging an attached outside process
-   rather than an inferior.  */
-
-int attach_flag;
-
 
 /* Record terminal status separately for debugger and inferior.  */
 
@@ -93,6 +94,7 @@ PROCESS_GROUP_TYPE inferior_process_group;
    inferior only.  If we have job control, that takes care of it.  If not,
    we save our handlers in these two variables and set SIGINT and SIGQUIT
    to SIG_IGN.  */
+
 static void (*sigint_ours) ();
 static void (*sigquit_ours) ();
 
@@ -140,6 +142,9 @@ gdb_has_a_terminal ()
 #ifdef HAVE_TERMIOS
 	      our_process_group = tcgetpgrp (0);
 #endif
+#ifdef HAVE_TERMIO
+	      our_process_group = getpgrp ();
+#endif
 #ifdef HAVE_SGTTY
 	      ioctl (0, TIOCGPGRP, &our_process_group);
 #endif
@@ -166,7 +171,8 @@ static void terminal_ours_1 PARAMS ((int));
    before we actually run the inferior.  */
 
 void
-terminal_init_inferior ()
+terminal_init_inferior_with_pgrp (pgrp)
+     int pgrp;
 {
   if (gdb_has_a_terminal ())
     {
@@ -175,18 +181,9 @@ terminal_init_inferior ()
       if (inferior_ttystate)
 	free (inferior_ttystate);
       inferior_ttystate = SERIAL_GET_TTY_STATE (stdin_serial);
+
 #ifdef PROCESS_GROUP_TYPE
-#ifdef PIDGET
-      /* This is for Lynx, and should be cleaned up by having Lynx be
-	 a separate debugging target with a version of
-	 target_terminal_init_inferior which passes in the process
-	 group to a generic routine which does all the work (and the
-	 non-threaded child_terminal_init_inferior can just pass in
-	 inferior_pid to the same routine).  */
-      inferior_process_group = PIDGET (inferior_pid);
-#else
-      inferior_process_group = inferior_pid;
-#endif
+      inferior_process_group = pgrp;
 #endif
 
       /* Make sure that next time we call terminal_inferior (which will be
@@ -196,6 +193,20 @@ terminal_init_inferior ()
     }
 }
 
+void
+terminal_init_inferior ()
+{
+#ifdef PROCESS_GROUP_TYPE
+  /* This is for Lynx, and should be cleaned up by having Lynx be a separate
+     debugging target with a version of target_terminal_init_inferior which
+     passes in the process group to a generic routine which does all the work
+     (and the non-threaded child_terminal_init_inferior can just pass in
+     inferior_pid to the same routine).  */
+  /* We assume INFERIOR_PID is also the child's process group.  */
+  terminal_init_inferior_with_pgrp (PIDGET (inferior_pid));
+#endif /* PROCESS_GROUP_TYPE */
+}
+
 /* Put the inferior's terminal settings into effect.
    This is preparation for starting or resuming the inferior.  */
 
@@ -225,7 +236,9 @@ terminal_inferior ()
       if (!job_control)
 	{
 	  sigint_ours = (void (*) ()) signal (SIGINT, SIG_IGN);
+#ifdef SIGQUIT
 	  sigquit_ours = (void (*) ()) signal (SIGQUIT, SIG_IGN);
+#endif
 	}
 
       /* If attach_flag is set, we don't know whether we are sharing a
@@ -318,13 +331,19 @@ terminal_ours_1 (output_only)
 #ifdef HAVE_TERMIOS
       inferior_process_group = tcgetpgrp (0);
 #endif
+#ifdef HAVE_TERMIO
+      inferior_process_group = getpgrp ();
+#endif
 #ifdef HAVE_SGTTY
       ioctl (0, TIOCGPGRP, &inferior_process_group);
 #endif
 
       /* Here we used to set ICANON in our ttystate, but I believe this
 	 was an artifact from before when we used readline.  Readline sets
-	 the tty state when it needs to.  */
+	 the tty state when it needs to.
+	 FIXME-maybe: However, query() expects non-raw mode and doesn't
+	 use readline.  Maybe query should use readline (on the other hand,
+	 this only matters for HAVE_SGTTY, not termio or termios, I think).  */
 
       /* Set tty state to our_ttystate.  We don't change in our out of raw
 	 mode, to avoid flushing input.  We need to do the same thing
@@ -332,6 +351,7 @@ terminal_ours_1 (output_only)
 	 terminal_is_ours and terminal_is_ours_for_output flags.  It's OK,
 	 though, since readline will deal with raw mode when/if it needs to.
 	 */
+
       SERIAL_NOFLUSH_SET_TTY_STATE (stdin_serial, our_ttystate,
 				    inferior_ttystate);
 
@@ -363,7 +383,9 @@ terminal_ours_1 (output_only)
       if (!job_control)
 	{
 	  signal (SIGINT, sigint_ours);
+#ifdef SIGQUIT
 	  signal (SIGQUIT, sigquit_ours);
+#endif
 	}
 
 #ifdef F_GETFL
@@ -485,7 +507,7 @@ new_tty ()
 
   if (inferior_thisrun_terminal == 0)
     return;
-#if !defined(__GO32__)
+#if !defined(__GO32__) && !defined(_WIN32)
 #ifdef TIOCNOTTY
   /* Disconnect the child process from our controlling terminal.  On some
      systems (SVR4 for example), this may cause a SIGTTOU, so temporarily
@@ -524,7 +546,7 @@ new_tty ()
     { close (2); dup (tty); }
   if (tty > 2)
     close(tty);
-#endif /* !go32 */
+#endif /* !go32 && !win32*/
 }
 
 /* Kill the inferior process.  Make us have no inferior.  */
@@ -535,7 +557,10 @@ kill_command (arg, from_tty)
      char *arg;
      int from_tty;
 {
-  /* Shouldn't this be target_has_execution?  FIXME.  */
+  /* FIXME:  This should not really be inferior_pid (or target_has_execution).
+     It should be a distinct flag that indicates that a target is active, cuz
+     some targets don't have processes! */
+
   if (inferior_pid == 0)
     error ("The program is not being run.");
   if (!query ("Kill the program being debugged? "))
@@ -547,36 +572,13 @@ kill_command (arg, from_tty)
   /* Killing off the inferior can leave us with a core file.  If so,
      print the state we are left in.  */
   if (target_has_stack) {
-    printf_filtered ("In %s,\n", current_target->to_longname);
+    printf_filtered ("In %s,\n", target_longname);
     if (selected_frame == NULL)
       fputs_filtered ("No selected stack frame.\n", gdb_stdout);
     else
       print_stack_frame (selected_frame, selected_frame_level, 1);
   }
 }
-
-/* The inferior process has died.  Long live the inferior!  */
-
-void
-generic_mourn_inferior ()
-{
-  inferior_pid = 0;
-  attach_flag = 0;
-  breakpoint_init_inferior ();
-  registers_changed ();
-
-#ifdef CLEAR_DEFERRED_STORES
-  /* Delete any pending stores to the inferior... */
-  CLEAR_DEFERRED_STORES;
-#endif
-
-  reopen_exec_file ();
-  reinit_frame_cache ();
-
-  /* It is confusing to the user for ignore counts to stick around
-     from previous runs of the inferior.  So clear them.  */
-  breakpoint_clear_ignore_counts ();
-}
 
 /* Call set_sigint_trap when you need to pass a signal on to an attached
    process when handling SIGINT */
@@ -586,7 +588,9 @@ static void
 pass_signal (signo)
     int signo;
 {
-  kill (inferior_pid, SIGINT);
+#ifndef _WIN32
+  kill (PIDGET (inferior_pid), SIGINT);
+#endif
 }
 
 static void (*osig)();
@@ -594,17 +598,84 @@ static void (*osig)();
 void
 set_sigint_trap()
 {
-  osig = (void (*) ()) signal (SIGINT, pass_signal);
+  if (attach_flag || inferior_thisrun_terminal)
+    {
+      osig = (void (*) ()) signal (SIGINT, pass_signal);
+    }
 }
 
 void
 clear_sigint_trap()
 {
-  signal (SIGINT, osig);
+  if (attach_flag || inferior_thisrun_terminal)
+    {
+      signal (SIGINT, osig);
+    }
 }
 
+#if defined (SIGIO) && defined (FASYNC) && defined (FD_SET) && defined (F_SETOWN)
+static void (*old_sigio) ();
 
-int job_control;
+static void
+handle_sigio (signo)
+     int signo;
+{
+  int numfds;
+  fd_set readfds;
+
+  signal (SIGIO, handle_sigio);
+
+  FD_ZERO (&readfds);
+  FD_SET (target_activity_fd, &readfds);
+  numfds = select (target_activity_fd + 1, &readfds, NULL, NULL, NULL);
+  if (numfds >= 0 && FD_ISSET (target_activity_fd, &readfds))
+    {
+#ifndef _WIN32
+      if ((*target_activity_function) ())
+	kill (inferior_pid, SIGINT);
+#endif
+    }
+}
+
+static int old_fcntl_flags;
+
+void
+set_sigio_trap ()
+{
+  if (target_activity_function)
+    {
+      old_sigio = (void (*) ()) signal (SIGIO, handle_sigio);
+      fcntl (target_activity_fd, F_SETOWN, getpid()); 
+      old_fcntl_flags = fcntl (target_activity_fd, F_GETFL, 0);
+      fcntl (target_activity_fd, F_SETFL, old_fcntl_flags | FASYNC);
+    }
+}
+
+void
+clear_sigio_trap ()
+{
+  if (target_activity_function)
+    {
+      signal (SIGIO, old_sigio);
+      fcntl (target_activity_fd, F_SETFL, old_fcntl_flags);
+    }
+}
+#else /* No SIGIO.  */
+void
+set_sigio_trap ()
+{
+  if (target_activity_function)
+    abort ();
+}
+
+void
+clear_sigio_trap ()
+{
+  if (target_activity_function)
+    abort ();
+}
+#endif /* No SIGIO.  */
+
 
 /* This is here because this is where we figure out whether we (probably)
    have job control.  Just using job_control only does part of it because
@@ -619,10 +690,10 @@ int
 gdb_setpgid ()
 {
   int retval = 0;
+
   if (job_control)
     {
-#if defined (NEED_POSIX_SETPGID) || defined (HAVE_TERMIOS)
-      /* Do all systems with termios have setpgid?  I hope so.  */
+#if defined (NEED_POSIX_SETPGID) || (defined (HAVE_TERMIOS) && defined (HAVE_SETPGID))
       /* setpgid (0, 0) is supposed to work and mean the same thing as
 	 this, but on Ultrix 4.2A it fails with EPERM (and
 	 setpgid (getpid (), getpid ()) succeeds).  */
@@ -662,9 +733,13 @@ _initialize_inflow ()
 #ifdef _POSIX_JOB_CONTROL
   job_control = 1;
 #else
+#ifdef _SC_JOB_CONTROL
   job_control = sysconf (_SC_JOB_CONTROL);
-#endif
-#endif /* termios */
+#else
+  job_control = 0;	/* have to assume the worst */
+#endif	/* _SC_JOB_CONTROL */
+#endif	/* _POSIX_JOB_CONTROL */
+#endif	/* HAVE_TERMIOS */
 
 #ifdef HAVE_SGTTY
 #ifdef TIOCGPGRP
