@@ -1,5 +1,5 @@
 /* Shared code to pre-read a stab (dbx-style), when building a psymtab.
-   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993
+   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996
    Free Software Foundation, Inc.
 
 This file is part of GDB.
@@ -16,12 +16,23 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* The following need to be defined:
    SET_NAMESTRING() --Set namestring to name of symbol.
    CUR_SYMBOL_TYPE --Type code of current symbol.
    CUR_SYMBOL_VALUE --Value field of current symbol.  May be adjusted here.
+   namestring - variable pointing to the name of the stab.
+   section_offsets - variable pointing to the section offsets.
+   pst - the partial symbol table being built.
+
+   psymtab_include_list, includes_used, includes_allocated - list of include
+     file names (N_SOL) seen so far.
+   dependency_list, dependencies_used, dependencies_allocated - list of
+     N_EXCL stabs seen so far.
+
+   END_PSYMTAB -- end a partial symbol table.
+   START_PSYMTAB -- start a partial symbol table.
  */
 
 /* End of macro definitions, now let's handle them symbols!  */
@@ -78,29 +89,24 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	  SET_NAMESTRING();
 	  if ((namestring[0] == '-' && namestring[1] == 'l')
 	      || (namestring [(nsl = strlen (namestring)) - 1] == 'o'
-		  && namestring [nsl - 2] == '.')
-#ifdef GDB_TARGET_IS_HPPA
-              /* some cooperation from gcc to get around ld stupidity */
-              || (namestring[0] == 'e' && STREQ (namestring, "end_file."))
-#endif
-	      )
+		  && namestring [nsl - 2] == '.'))
 	    {
-#ifndef GDB_TARGET_IS_HPPA
 	      if (objfile -> ei.entry_point <  CUR_SYMBOL_VALUE &&
 		  objfile -> ei.entry_point >= last_o_file_start)
 		{
 		  objfile -> ei.entry_file_lowpc = last_o_file_start;
 		  objfile -> ei.entry_file_highpc = CUR_SYMBOL_VALUE;
 		}
-#endif
 	      if (past_first_source_file && pst
 		  /* The gould NP1 uses low values for .o and -l symbols
 		     which are not the address.  */
 		  && CUR_SYMBOL_VALUE >= pst->textlow)
 		{
 		  END_PSYMTAB (pst, psymtab_include_list, includes_used,
-			       symnum * symbol_size, CUR_SYMBOL_VALUE,
-			       dependency_list, dependencies_used);
+			       symnum * symbol_size,
+			       CUR_SYMBOL_VALUE > pst->texthigh
+				 ? CUR_SYMBOL_VALUE : pst->texthigh, 
+			       dependency_list, dependencies_used, textlow_not_set);
 		  pst = (struct partial_symtab *) 0;
 		  includes_used = 0;
 		  dependencies_used = 0;
@@ -139,7 +145,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 	case N_UNDF:
 #ifdef DBXREAD_ONLY
-	  if (processing_acc_compilation && bufp->n_strx == 1) {
+	  if (processing_acc_compilation && CUR_SYMBOL_STRX == 1) {
 	    /* Deal with relative offsets in the string table
 	       used in ELF+STAB under Solaris.  If we want to use the
 	       n_strx field, which contains the name of the file,
@@ -148,7 +154,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	    past_first_source_file = 1;
 	    file_string_table_offset = next_file_string_table_offset;
 	    next_file_string_table_offset =
-	      file_string_table_offset + bufp->n_value;
+	      file_string_table_offset + CUR_SYMBOL_VALUE;
 	    if (next_file_string_table_offset < file_string_table_offset)
 	      error ("string table offset backs up at %d", symnum);
   /* FIXME -- replace error() with complaint.  */
@@ -191,9 +197,27 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	  static int prev_so_symnum = -10;
 	  static int first_so_symnum;
 	  char *p;
+	  int prev_textlow_not_set;
 
 	  valu = CUR_SYMBOL_VALUE + ANOFFSET (section_offsets, SECT_OFF_TEXT);
 
+	  prev_textlow_not_set = textlow_not_set;
+
+#ifdef SOFUN_ADDRESS_MAYBE_MISSING
+	  /* A zero value is probably an indication for the SunPRO 3.0
+	     compiler. end_psymtab explicitly tests for zero, so
+	     don't relocate it.  */
+
+	  if (CUR_SYMBOL_VALUE == 0)
+	    {
+	      textlow_not_set = 1;
+	      valu = 0;
+	    }
+	  else
+	    textlow_not_set = 0;
+#else
+	  textlow_not_set = 0;
+#endif
 	  past_first_source_file = 1;
 
 	  if (prev_so_symnum != symnum - 1)
@@ -203,8 +227,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	      if (pst)
 		{
 		  END_PSYMTAB (pst, psymtab_include_list, includes_used,
-			       symnum * symbol_size, valu,
-			       dependency_list, dependencies_used);
+			       symnum * symbol_size,
+			       valu > pst->texthigh ? valu : pst->texthigh,
+			       dependency_list, dependencies_used,
+			       prev_textlow_not_set);
 		  pst = (struct partial_symtab *) 0;
 		  includes_used = 0;
 		  dependencies_used = 0;
@@ -217,6 +243,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 	  SET_NAMESTRING();
 
+	  /* Null name means end of .o file.  Don't start a new one. */
+	  if (*namestring == '\000')
+	    continue;
+
 	  /* Some compilers (including gcc) emit a pair of initial N_SOs.
 	     The first one is a directory name; the second the file name.
 	     If pst exists, is empty, and has a filename ending in '/',
@@ -240,69 +270,103 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	}
 
 	case N_BINCL:
+	  {
 #ifdef DBXREAD_ONLY
-	  /* Add this bincl to the bincl_list for future EXCLs.  No
-	     need to save the string; it'll be around until
-	     read_dbx_symtab function returns */
+	    enum language tmp_language;
+	    /* Add this bincl to the bincl_list for future EXCLs.  No
+	       need to save the string; it'll be around until
+	       read_dbx_symtab function returns */
 
-	  SET_NAMESTRING();
+	    SET_NAMESTRING();
 
-	  add_bincl_to_list (pst, namestring, CUR_SYMBOL_VALUE);
+	    tmp_language = deduce_language_from_filename (namestring);
 
-	  /* Mark down an include file in the current psymtab */
+	    /* Only change the psymtab's language if we've learned
+	       something useful (eg. tmp_language is not language_unknown).
+	       In addition, to match what start_subfile does, never change
+	       from C++ to C.  */
+	    if (tmp_language != language_unknown
+		&& (tmp_language != language_c
+		    || psymtab_language != language_cplus))
+	      psymtab_language = tmp_language;
 
-	  goto record_include_file;
+	    if (pst == NULL)
+	      {
+		/* FIXME: we should not get here without a PST to work on.
+		   Attempt to recover.  */
+		complain (&unclaimed_bincl_complaint, namestring, symnum);
+		continue;
+	      }
+	    add_bincl_to_list (pst, namestring, CUR_SYMBOL_VALUE);
+
+	    /* Mark down an include file in the current psymtab */
+
+	    goto record_include_file;
 
 #else /* DBXREAD_ONLY */
-	  continue;
-#endif
-
-	case N_SOL:
-	  /* Mark down an include file in the current psymtab */
-
-	  SET_NAMESTRING();
-
-	  /* In C++, one may expect the same filename to come round many
-	     times, when code is coming alternately from the main file
-	     and from inline functions in other files. So I check to see
-	     if this is a file we've seen before -- either the main
-	     source file, or a previously included file.
-
-	     This seems to be a lot of time to be spending on N_SOL, but
-	     things like "break c-exp.y:435" need to work (I
-	     suppose the psymtab_include_list could be hashed or put
-	     in a binary tree, if profiling shows this is a major hog).  */
-	  if (pst && STREQ (namestring, pst->filename))
 	    continue;
-	  {
-	    register int i;
-	    for (i = 0; i < includes_used; i++)
-	      if (STREQ (namestring, psymtab_include_list[i]))
-		{
-		  i = -1; 
-		  break;
-		}
-	    if (i == -1)
-	      continue;
+#endif
 	  }
 
-#ifdef DBXREAD_ONLY
-	record_include_file:
-#endif
-
-	  psymtab_include_list[includes_used++] = namestring;
-	  if (includes_used >= includes_allocated)
+	case N_SOL:
+	  {
+	    enum language tmp_language;
+	    /* Mark down an include file in the current psymtab */
+	    
+	    SET_NAMESTRING();
+  
+	    tmp_language = deduce_language_from_filename (namestring);
+  
+	    /* Only change the psymtab's language if we've learned
+	       something useful (eg. tmp_language is not language_unknown).
+	       In addition, to match what start_subfile does, never change
+	       from C++ to C.  */
+	    if (tmp_language != language_unknown
+		&& (tmp_language != language_c
+		    || psymtab_language != language_cplus))
+	      psymtab_language = tmp_language;
+	    
+	    /* In C++, one may expect the same filename to come round many
+	       times, when code is coming alternately from the main file
+	       and from inline functions in other files. So I check to see
+	       if this is a file we've seen before -- either the main
+	       source file, or a previously included file.
+	       
+	       This seems to be a lot of time to be spending on N_SOL, but
+	       things like "break c-exp.y:435" need to work (I
+	       suppose the psymtab_include_list could be hashed or put
+	       in a binary tree, if profiling shows this is a major hog).  */
+	    if (pst && STREQ (namestring, pst->filename))
+	      continue;
 	    {
-	      char **orig = psymtab_include_list;
-
-	      psymtab_include_list = (char **)
-		alloca ((includes_allocated *= 2) *
-			sizeof (char *));
-	      memcpy ((PTR)psymtab_include_list, (PTR)orig,
-		      includes_used * sizeof (char *));
+	      register int i;
+	      for (i = 0; i < includes_used; i++)
+		if (STREQ (namestring, psymtab_include_list[i]))
+		  {
+		    i = -1; 
+		    break;
+		  }
+	      if (i == -1)
+		continue;
 	    }
-	  continue;
-
+	    
+#ifdef DBXREAD_ONLY
+	  record_include_file:
+#endif
+	    
+	    psymtab_include_list[includes_used++] = namestring;
+	    if (includes_used >= includes_allocated)
+	      {
+		char **orig = psymtab_include_list;
+		
+		psymtab_include_list = (char **)
+		  alloca ((includes_allocated *= 2) *
+			  sizeof (char *));
+		memcpy ((PTR)psymtab_include_list, (PTR)orig,
+			includes_used * sizeof (char *));
+	      }
+	    continue;
+	  }
 	case N_LSYM:		/* Typedef or automatic variable. */
 	case N_STSYM:		/* Data seg var -- static  */
 	case N_LCSYM:		/* BSS      "  */
@@ -323,6 +387,22 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 	  SET_NAMESTRING();
 
+#ifdef DBXREAD_ONLY
+	  /* See if this is an end of function stab.  */
+	  if (CUR_SYMBOL_TYPE == N_FUN && *namestring == '\000')
+	    {
+	      unsigned long valu;
+
+	      /* It's value is the size (in bytes) of the function for
+		 function relative stabs, or the address of the function's
+		 end for old style stabs.  */
+	      valu = CUR_SYMBOL_VALUE + last_function_start;
+	      if (pst->texthigh == 0 || valu > pst->texthigh)
+		pst->texthigh = valu;
+	      break;
+	     }
+#endif
+
 	  p = (char *) strchr (namestring, ':');
 	  if (!p)
 	    continue;		/* Not a debugging symbol.   */
@@ -340,50 +420,67 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	    {
 	    case 'S':
 	      CUR_SYMBOL_VALUE += ANOFFSET (section_offsets, SECT_OFF_DATA);
-	      ADD_PSYMBOL_ADDR_TO_LIST (namestring, p - namestring,
-					VAR_NAMESPACE, LOC_STATIC,
-					objfile->static_psymbols,
-					CUR_SYMBOL_VALUE,
-					psymtab_language, objfile);
+#ifdef STATIC_TRANSFORM_NAME
+	      namestring = STATIC_TRANSFORM_NAME (namestring);
+#endif
+	      add_psymbol_to_list (namestring, p - namestring,
+				   VAR_NAMESPACE, LOC_STATIC,
+				   &objfile->static_psymbols,
+				   0, CUR_SYMBOL_VALUE,
+				   psymtab_language, objfile);
 	      continue;
 	    case 'G':
 	      CUR_SYMBOL_VALUE += ANOFFSET (section_offsets, SECT_OFF_DATA);
 	      /* The addresses in these entries are reported to be
 		 wrong.  See the code that reads 'G's for symtabs. */
-	      ADD_PSYMBOL_ADDR_TO_LIST (namestring, p - namestring,
-					VAR_NAMESPACE, LOC_STATIC,
-					objfile->global_psymbols,
-					CUR_SYMBOL_VALUE,
-					psymtab_language, objfile);
+	      add_psymbol_to_list (namestring, p - namestring,
+				   VAR_NAMESPACE, LOC_STATIC,
+				   &objfile->global_psymbols,
+				   0, CUR_SYMBOL_VALUE,
+				   psymtab_language, objfile);
 	      continue;
 
 	    case 'T':
 	      if (p != namestring)	/* a name is there, not just :T... */
 		{
-		  ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+		  add_psymbol_to_list (namestring, p - namestring,
 				       STRUCT_NAMESPACE, LOC_TYPEDEF,
-				       objfile->static_psymbols,
-				       CUR_SYMBOL_VALUE,
+				       &objfile->static_psymbols,
+				       CUR_SYMBOL_VALUE, 0,
 				       psymtab_language, objfile);
 		  if (p[2] == 't')
 		    {
 		      /* Also a typedef with the same name.  */
-		      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+		      add_psymbol_to_list (namestring, p - namestring,
 					   VAR_NAMESPACE, LOC_TYPEDEF,
-					   objfile->static_psymbols,
-					   CUR_SYMBOL_VALUE, psymtab_language,
-					   objfile);
+					   &objfile->static_psymbols,
+					   CUR_SYMBOL_VALUE, 0,
+					   psymtab_language, objfile);
 		      p += 1;
 		    }
+		  /* The semantics of C++ state that "struct foo { ... }"
+		     also defines a typedef for "foo".  Unfortuantely, cfront
+		     never makes the typedef when translating from C++ to C.
+		     We make the typedef here so that "ptype foo" works as
+		     expected for cfront translated code.  */
+		  else if (psymtab_language == language_cplus)
+		   {
+		      /* Also a typedef with the same name.  */
+		      add_psymbol_to_list (namestring, p - namestring,
+					   VAR_NAMESPACE, LOC_TYPEDEF,
+					   &objfile->static_psymbols,
+					   CUR_SYMBOL_VALUE, 0,
+					   psymtab_language, objfile);
+		   }
 		}
 	      goto check_enum;
 	    case 't':
 	      if (p != namestring)	/* a name is there, not just :T... */
 		{
-		  ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+		  add_psymbol_to_list (namestring, p - namestring,
 				       VAR_NAMESPACE, LOC_TYPEDEF,
-				       objfile->static_psymbols,
-				       CUR_SYMBOL_VALUE,
+				       &objfile->static_psymbols,
+				       CUR_SYMBOL_VALUE, 0,
 				       psymtab_language, objfile);
 		}
 	    check_enum:
@@ -411,6 +508,17 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 	      if (*p++ == 'e')
 		{
+		  /* The aix4 compiler emits extra crud before the members.  */
+		  if (*p == '-')
+		    {
+		      /* Skip over the type (?).  */
+		      while (*p != ':')
+			p++;
+
+		      /* Skip over the colon.  */
+		      p++;
+		    }
+
 		  /* We have found an enumerated type.  */
 		  /* According to comments in read_enum_type
 		     a comma could end it instead of a semicolon.
@@ -422,8 +530,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 		      /* Check for and handle cretinous dbx symbol name
 			 continuation!  */
-		      if (*p == '\\')
-			p = next_symbol_text ();
+		      if (*p == '\\' || (*p == '?' && p[1] == '\0'))
+			p = next_symbol_text (objfile);
 
 		      /* Point to the character after the name
 			 of the enum constant.  */
@@ -431,10 +539,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 			;
 		      /* Note that the value doesn't matter for
 			 enum constants in psymtabs, just in symtabs.  */
-		      ADD_PSYMBOL_TO_LIST (p, q - p,
+		      add_psymbol_to_list (p, q - p,
 					   VAR_NAMESPACE, LOC_CONST,
-					   objfile->static_psymbols, 0,
-					   psymtab_language, objfile);
+					   &objfile->static_psymbols, 0,
+					   0, psymtab_language, objfile);
 		      /* Point past the name.  */
 		      p = q;
 		      /* Skip over the value.  */
@@ -448,49 +556,98 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	      continue;
 	    case 'c':
 	      /* Constant, e.g. from "const" in Pascal.  */
-	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+	      add_psymbol_to_list (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_CONST,
-				   objfile->static_psymbols, CUR_SYMBOL_VALUE,
-				   psymtab_language, objfile);
+				   &objfile->static_psymbols, CUR_SYMBOL_VALUE,
+				   0, psymtab_language, objfile);
 	      continue;
 
 	    case 'f':
+	      CUR_SYMBOL_VALUE += ANOFFSET (section_offsets, SECT_OFF_TEXT);
 #ifdef DBXREAD_ONLY
+	      /* Keep track of the start of the last function so we
+		 can handle end of function symbols.  */
+	      last_function_start = CUR_SYMBOL_VALUE;
 	      /* Kludges for ELF/STABS with Sun ACC */
 	      last_function_name = namestring;
-	      if (pst && pst->textlow == 0)
-		pst->textlow = CUR_SYMBOL_VALUE;
+#ifdef SOFUN_ADDRESS_MAYBE_MISSING
+	      /* Do not fix textlow==0 for .o or NLM files, as 0 is a legit
+		 value for the bottom of the text seg in those cases. */
+	      if (pst && textlow_not_set)
+		{
+		  pst->textlow =
+		    find_stab_function_addr (namestring, pst, objfile);
+		  textlow_not_set = 0;
+		}
+#endif
 #if 0
 	      if (startup_file_end == 0)
 		startup_file_end = CUR_SYMBOL_VALUE;
 #endif
 	      /* End kludge.  */
+
+	      /* In reordered executables this function may lie outside
+		 the bounds created by N_SO symbols.  If that's the case
+		 use the address of this function as the low bound for
+		 the partial symbol table.  */
+	      if (textlow_not_set
+		  || (CUR_SYMBOL_VALUE < pst->textlow
+		      && CUR_SYMBOL_VALUE
+			   != ANOFFSET (section_offsets, SECT_OFF_TEXT)))
+		{
+		  pst->textlow = CUR_SYMBOL_VALUE;
+		  textlow_not_set = 0;
+		}
 #endif /* DBXREAD_ONLY */
-	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+	      add_psymbol_to_list (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_BLOCK,
-				   objfile->static_psymbols, CUR_SYMBOL_VALUE,
-				   psymtab_language, objfile);
+				   &objfile->static_psymbols, CUR_SYMBOL_VALUE,
+				   0, psymtab_language, objfile);
 	      continue;
 
 	      /* Global functions were ignored here, but now they
 	         are put into the global psymtab like one would expect.
 		 They're also in the minimal symbol table.  */
 	    case 'F':
+	      CUR_SYMBOL_VALUE += ANOFFSET (section_offsets, SECT_OFF_TEXT);
 #ifdef DBXREAD_ONLY
+	      /* Keep track of the start of the last function so we
+		 can handle end of function symbols.  */
+	      last_function_start = CUR_SYMBOL_VALUE;
 	      /* Kludges for ELF/STABS with Sun ACC */
 	      last_function_name = namestring;
-	      if (pst && pst->textlow == 0)
-		pst->textlow = CUR_SYMBOL_VALUE;
+#ifdef SOFUN_ADDRESS_MAYBE_MISSING
+	      /* Do not fix textlow==0 for .o or NLM files, as 0 is a legit
+		 value for the bottom of the text seg in those cases. */
+	      if (pst && textlow_not_set)
+		{
+		  pst->textlow =
+		    find_stab_function_addr (namestring, pst, objfile);
+		  textlow_not_set = 0;
+		}
+#endif
 #if 0
 	      if (startup_file_end == 0)
 		startup_file_end = CUR_SYMBOL_VALUE;
 #endif
 	      /* End kludge.  */
+	      /* In reordered executables this function may lie outside
+		 the bounds created by N_SO symbols.  If that's the case
+		 use the address of this function as the low bound for
+		 the partial symbol table.  */
+	      if (textlow_not_set
+		  || (CUR_SYMBOL_VALUE < pst->textlow
+		      && CUR_SYMBOL_VALUE
+			   != ANOFFSET (section_offsets, SECT_OFF_TEXT)))
+		{
+		  pst->textlow = CUR_SYMBOL_VALUE;
+		  textlow_not_set = 0;
+		}
 #endif /* DBXREAD_ONLY */
-	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+	      add_psymbol_to_list (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_BLOCK,
-				   objfile->global_psymbols, CUR_SYMBOL_VALUE,
-				   psymtab_language, objfile);
+				   &objfile->global_psymbols, CUR_SYMBOL_VALUE,
+				   0, psymtab_language, objfile);
 	      continue;
 
 	      /* Two things show up here (hopefully); static symbols of
@@ -508,16 +665,35 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	    case '7':
 	    case '8':
 	    case '9':
+	    case '-':
+	    case '#':   /* for symbol identification (used in live ranges) */
+	   /* added to support cfront stabs strings */
+	    case 'Z':	/* for definition continuations */
+	    case 'P':	/* for prototypes */
+	      continue;
+
+	    case ':':
+	      /* It is a C++ nested symbol.  We don't need to record it
+		 (I don't think); if we try to look up foo::bar::baz,
+		 then symbols for the symtab containing foo should get
+		 read in, I think.  */
+	      /* Someone says sun cc puts out symbols like
+		 /foo/baz/maclib::/usr/local/bin/maclib,
+		 which would get here with a symbol type of ':'.  */
 	      continue;
 
 	    default:
-	      /* Unexpected symbol.  Ignore it; perhaps it is an extension
-		 that we don't know about.
+	      /* Unexpected symbol descriptor.  The second and subsequent stabs
+		 of a continued stab can show up here.  The question is
+		 whether they ever can mimic a normal stab--it would be
+		 nice if not, since we certainly don't want to spend the
+		 time searching to the end of every string looking for
+		 a backslash.  */
 
-		 Someone says sun cc puts out symbols like
-		 /foo/baz/maclib::/usr/local/bin/maclib,
-		 which would get here with a symbol type of ':'.  */
 	      complain (&unknown_symchar_complaint, p[1]);
+
+	      /* Ignore it; perhaps it is an extension that we don't
+		 know about.  */
 	      continue;
 	    }
 
@@ -563,19 +739,35 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 			   (dependencies_used
 			    * sizeof (struct partial_symtab *)));
 #ifdef DEBUG_INFO
-		    fprintf (stderr, "Had to reallocate dependency list.\n");
-		    fprintf (stderr, "New dependencies allocated: %d\n",
+		    fprintf_unfiltered (gdb_stderr, "Had to reallocate dependency list.\n");
+		    fprintf_unfiltered (gdb_stderr, "New dependencies allocated: %d\n",
 			     dependencies_allocated);
 #endif
 		  }
 	      }
-	    else
-	      error ("Invalid symbol data: \"repeated\" header file not previously seen, at symtab pos %d.",
-		     symnum);
 	  }
 #endif /* DBXREAD_ONLY */
 	  continue;
 
+	case N_ENDM:
+#ifdef SOFUN_ADDRESS_MAYBE_MISSING
+	  /* Solaris 2 end of module, finish current partial symbol table.
+	     END_PSYMTAB will set pst->texthigh to the proper value, which
+	     is necessary if a module compiled without debugging info
+	     follows this module.  */
+	  if (pst)
+	    {
+	      END_PSYMTAB (pst, psymtab_include_list, includes_used,
+			   symnum * symbol_size,
+			   (CORE_ADDR) 0,
+			   dependency_list, dependencies_used, textlow_not_set);
+	      pst = (struct partial_symtab *) 0;
+	      includes_used = 0;
+	      dependencies_used = 0;
+	    }
+#endif
+	  continue;
+
 	case N_RBRAC:
 #ifdef HANDLE_RBRAC
 	  HANDLE_RBRAC(CUR_SYMBOL_VALUE);
@@ -604,7 +796,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 	case N_OBJ:		/* useless types from Solaris */
 	case N_OPT:
-	case N_ENDM:
 	  /* These symbols aren't interesting; don't worry about them */
 
 	  continue;
@@ -613,6 +804,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	  /* If we haven't found it yet, ignore it.  It's probably some
 	     new type we don't know about yet.  */
 	  complain (&unknown_symtype_complaint,
-		    local_hex_string ((unsigned long) CUR_SYMBOL_TYPE));
+		    local_hex_string (CUR_SYMBOL_TYPE));
 	  continue;
 	}
