@@ -1,5 +1,5 @@
 /* Shared library declarations for GDB, the GNU Debugger.
-   Copyright (C) 1992 Free Software Foundation, Inc.
+   Copyright (C) 1992, 1998 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef __STDC__		/* Forward decl's for prototypes */
 struct target_ops;
@@ -50,8 +50,23 @@ solib_create_inferior_hook PARAMS((void));	/* solib.c */
 /* If we can't set a breakpoint, and it's in a shared library, just
    disable it.  */
 
-#define DISABLE_UNSETTABLE_BREAK(addr)	solib_address(addr)
+#define DISABLE_UNSETTABLE_BREAK(addr)	(solib_address(addr) != NULL)
 
-extern int
+extern char *
 solib_address PARAMS ((CORE_ADDR));		/* solib.c */
 
+/* If ADDR lies in a shared library, return its name.  */
+
+#define PC_SOLIB(addr)	solib_address (addr)
+
+#ifdef SVR4_SHARED_LIBS
+
+/* Return 1 if PC lies in the dynamic symbol resolution code of the
+   SVR4 run time loader.  */
+
+#define IN_SOLIB_DYNSYM_RESOLVE_CODE(pc) in_svr4_dynsym_resolve_code (pc)
+
+extern int
+in_svr4_dynsym_resolve_code PARAMS ((CORE_ADDR));
+
+#endif
