@@ -22,6 +22,8 @@ code), but it is a complete example. Defining NOOS will disable all the
 fun, and definign VAILDATE will define SWI 1 to enter SVC mode, and SWI
 0x11 to halt the emulator. */
 
+#include "config.h"
+
 #include <time.h>
 #include <errno.h>
 #include <string.h>
@@ -34,7 +36,7 @@ fun, and definign VAILDATE will define SWI 1 to enter SVC mode, and SWI
 #define unlink(s) remove(s)
 #endif
 
-#ifdef __sun__
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>	/* For SEEK_SET etc */
 #endif
 
@@ -65,6 +67,9 @@ extern int _fisatty(FILE *);
 #endif
 #endif
 
+/* For RDIError_BreakpointReached.  */
+#include "dbg_rdi.h"
+
 extern unsigned ARMul_OSInit(ARMul_State *state) ;
 extern void ARMul_OSExit(ARMul_State *state) ;
 extern unsigned ARMul_OSHandleSWI(ARMul_State *state,ARMword number) ;
@@ -231,12 +236,11 @@ unsigned ARMul_OSHandleSWI(ARMul_State *state,ARMword number)
 	int i;
 	char *local = malloc (len);
 	res = read (f,local, len);
-	for (i = 0; i < len; i++) 
-	  {
+	if (res > 0)
+	  for (i = 0; i < res; i++) 
 	    ARMul_WriteByte(state, ptr + i, local[i]) ;
-	  }
 	free (local);
-	state->Reg[0] = res;
+	state->Reg[0] = res == -1 ? -1 : len - res;
 	OSptr->ErrorNo = errno;
 	return TRUE;     
       }
@@ -253,7 +257,8 @@ unsigned ARMul_OSHandleSWI(ARMul_State *state,ARMword number)
 	  {
 	    local[i] = ARMul_ReadByte(state, ptr + i);
 	  }
-	state->Reg[0] = write (f, local, len);
+	res = write (f, local, len);
+	state->Reg[0] = res == -1 ? -1 : len - res;
 	free (local);
 	OSptr->ErrorNo = errno;
 	return TRUE;     
@@ -295,10 +300,39 @@ unsigned ARMul_OSHandleSWI(ARMul_State *state,ARMword number)
       OSptr->ErrorNo = errno ;
       return(TRUE) ;
 
+    case SWI_Write0 :
+      addr = state->Reg[0] ;
+      while ((temp = ARMul_ReadByte(state,addr++)) != 0)
+        (void)fputc((char)temp,stderr) ;
+      OSptr->ErrorNo = errno ;
+      return(TRUE) ;
+
     case SWI_GetErrno :
       state->Reg[0] = OSptr->ErrorNo ;
       return(TRUE) ;
 
+    case SWI_Breakpoint :
+      state->EndCondition = RDIError_BreakpointReached ;
+      state->Emulate = FALSE ;
+      return(TRUE) ;
+
+    case SWI_GetEnv :
+       state->Reg[0] = ADDRCMDLINE ;
+       if (state->MemSize)
+          state->Reg[1] = state->MemSize ;
+       else
+          state->Reg[1] = ADDRUSERSTACK ;
+
+       addr = state->Reg[0] ;
+       cptr = state->CommandLine ;
+       if (cptr == NULL)
+          cptr = "\0" ;
+       do {
+          temp = (ARMword)*cptr++ ;
+          ARMul_WriteByte(state,addr++,temp) ;
+          } while (temp != 0) ;
+       return(TRUE) ;
+
     default :
       state->Emulate = FALSE ;      
       return(FALSE) ;
