@@ -15,8 +15,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Z8KSIM; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+along with Z8KZIM; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* This program generates the code which emulates each of the z8k
    instructions
@@ -43,8 +43,20 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* steve chamberlain
    sac@cygnus.com */
 
+#include "config.h"
+
 #include <ansidecl.h>
-#include "sysdep.h"
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 
 #define NICENAMES
 
@@ -611,7 +623,15 @@ break;
 	    break;
 
 	case CLASS_DISP8:
+	    /* We can't use `(char)' since char might be unsigned.
+	       We can't use `(signed char)' because the compiler might be K&R.
+	       This seems safe, since it only assumes that bytes are 8
+	       bits.  */
+	    emit ("register unsigned int oplval_dst=((ibytes_1 << (sizeof (int) * 8 - 8)) >> (sizeof (int) * 8 - 9)) + pc;\n");
+#if 0
+	    /* Original code: fails if characters are unsigned.  */
 	    emit ("register unsigned int oplval_dst=(((char)ibytes_1)<<1) + pc;\n");
+#endif
 	    nibs += 2;
 	    break;
 	case CLASS_CC:
@@ -661,6 +681,7 @@ info_special (p, getdst, nostore, before, nosrc)
     case OPC_com:
     case OPC_comb:
     case OPC_adc:
+    case OPC_sbc:
     case OPC_nop:
     case OPC_adcb:
     case OPC_add:
@@ -863,12 +884,12 @@ info_fetch (p, getdst)
 }
 
 static void
-normal_flags (p, s)
+normal_flags (p, s, neg)
      opcode_entry_type *p;
      char *s;
 {
   emit (" %s;\n", s);
-  emit ("NORMAL_FLAGS(context,%d, tmp,  op_dst, op_src); \n", p->type);
+  emit ("NORMAL_FLAGS(context,%d, tmp,  op_dst, op_src,%d); \n", p->type,neg);
 }
 
 static void
@@ -891,7 +912,7 @@ test_normal_flags (p, s, opt)
 }
 
 static void
-optimize_normal_flags (p, s)
+optimize_normal_flags (p, s,neg)
      opcode_entry_type *p;
      char *s;
 {
@@ -899,7 +920,7 @@ optimize_normal_flags (p, s)
 #if 0
   emit ("context->broken_flags = CMP_FLAGS;\n");
 #else
-  emit ("NORMAL_FLAGS(context,%d, tmp,  op_dst, op_src); \n", p->type);
+  emit ("NORMAL_FLAGS(context,%d, tmp,  op_dst, op_src,%d); \n", p->type, neg);
 #endif
 }
 
@@ -1010,7 +1031,14 @@ shift (p, arith)
      int arith;
 {
 
+  /* We can't use `(char)' since char might be unsigned.
+     We can't use `(signed char)' because the compiler might be K&R.
+     This seems safe, since it only assumes that bytes are 8 bits.  */
+  emit ("op_src = (op_src << (sizeof (int) * 8 - 8)) >> (sizeof (int) * 8 - 8);\n");
+#if 0
+  /* Original code: fails if characters are unsigned.  */
   emit ("op_src = (char)op_src;\n");
+#endif
   emit ("if (op_src < 0) \n");
   emit ("{\n");
   emit ("op_src = -op_src;\n");
@@ -1043,16 +1071,16 @@ rotate (p, through_carry, size, left)
     {
       emit ("while (op_src--) {\n");
       emit ("int rotbit;\n");
-      emit ("rotbit = tmp & 1;\n");
-      emit ("tmp = ((unsigned)op_dst) >> op_src;\n");
+      emit ("rotbit = op_dst & 1;\n");
+      emit ("op_dst = ((unsigned)op_dst) >> 1;\n");
 
       if (through_carry)
 	{
-	  emit ("tmp |= context->carry << %d;\n", size - 1);
+	  emit ("op_dst |= context->carry << %d;\n", size - 1);
 	}
       else
 	{
-	  emit ("tmp |= rotbit << %d;\n", size - 1);
+	  emit ("op_dst |= rotbit << %d;\n", size - 1);
 	}
       emit ("context->carry = rotbit;\n");
       emit ("}\n");
@@ -1097,18 +1125,27 @@ adiv (p)
 
   if (p->type == 32)
     {
-      emit ("op_dst.low = get_long_reg(context,reg_dst+2);\n");
-      emit ("op_dst.high = get_long_reg(context,reg_dst+0);\n");
-      emit ("tmp = op_dst.low / op_src;\n");
+      emit ("op_dst.low = (int)get_long_reg(context,reg_dst+2);\n");
+      emit ("op_dst.high = (int)get_long_reg(context,reg_dst+0);\n");
+#ifdef __GNUC__
+      emit ("tmp = (((long long)op_dst.high << 32) + (op_dst.low)) / (int)op_src;\n");
+#else
+      emit ("tmp = (long)op_dst.low / (int)op_src;\n");
+#endif
       emit ("put_long_reg(context,reg_dst+2, tmp);\n");
-      emit ("put_long_reg(context,reg_dst, op_dst.low %% op_src);\n");
+#ifdef __GNUC__
+      emit ("put_long_reg(context,reg_dst, (((long long)op_dst.high << 32) + (op_dst.low)) %% (int)op_src);\n");
+#else
+      emit ("put_long_reg(context,reg_dst, (int)op_dst.low %% (int)op_src);\n");
+#endif
+
       emit ("context->zero = op_src == 0 || (op_dst.low==0 && op_dst.high==0);\n");
     }
   else
     {
-      emit ("tmp = op_dst / op_src;\n");
+      emit ("tmp = (long)op_dst / (short)op_src;\n");
       emit ("put_word_reg(context,reg_dst+1, tmp);\n");
-      emit ("put_word_reg(context,reg_dst,  op_dst %% op_src);\n");
+      emit ("put_word_reg(context,reg_dst, (long) op_dst %% (short)op_src);\n");
       emit ("context->zero = op_src == 0 || op_dst==0;\n");
     }
 
@@ -1249,7 +1286,10 @@ info_docode (p)
       break;
     case OPC_adc:
     case OPC_adcb:
-      optimize_normal_flags (p, "tmp = op_dst + op_src + PSW_CARRY");
+      normal_flags (p, "op_src += COND(context,7);tmp = op_dst + op_src ;",0);
+      break;
+    case OPC_sbc:
+      normal_flags (p, "op_src +=  COND(context,7);tmp = op_dst - op_src ;",1);
       break;
     case OPC_nop:
       break;
@@ -1300,6 +1340,7 @@ info_docode (p)
       break;
     case OPC_rrc:
       rotate (p, 1, 16, 0);
+      break;
     case OPC_rrcb:
       rotate (p, 1, 8, 0);
       break;
@@ -1320,7 +1361,7 @@ info_docode (p)
     case OPC_addl:
     case OPC_inc:
     case OPC_incb:
-      optimize_normal_flags (p, "tmp = op_dst + op_src");
+      optimize_normal_flags (p, "tmp = op_dst + op_src",0);
       break;
     case OPC_testb:
     case OPC_test:
@@ -1330,14 +1371,14 @@ info_docode (p)
     case OPC_cp:
     case OPC_cpb:
     case OPC_cpl:
-      normal_flags (p, "op_src = -op_src; tmp = op_dst + op_src");
+      normal_flags (p, "tmp = op_dst - op_src",1);
       break;
     case OPC_negb:
     case OPC_neg:
       emit ("{\n");
       emit ("int op_src = -op_dst;\n");
       emit ("op_dst = 0;\n");
-      optimize_normal_flags (p, "tmp = op_dst + op_src;\n");
+      optimize_normal_flags (p, "tmp = op_dst + op_src;\n",1);
       emit ("}");
       break;
 
@@ -1346,7 +1387,7 @@ info_docode (p)
     case OPC_subl:
     case OPC_dec:
     case OPC_decb:
-      optimize_normal_flags (p, "op_src = -op_src ;tmp = op_dst + op_src");
+      optimize_normal_flags (p, "tmp = op_dst - op_src",1);
       break;
     case OPC_bpt:
       bpt ();
